/****************************************************************************
** Meta object code from reading C++ file 'recorddialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "recorddialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'recorddialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RecordDialog_t {
    QByteArrayData data[11];
    char stringdata0[227];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RecordDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RecordDialog_t qt_meta_stringdata_RecordDialog = {
    {
QT_MOC_LITERAL(0, 0, 12), // "RecordDialog"
QT_MOC_LITERAL(1, 13, 28), // "deltaUnitsComboBox_activated"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 27), // "selectedRadioButton_clicked"
QT_MOC_LITERAL(4, 71, 29), // "allGadgetsRadioButton_clicked"
QT_MOC_LITERAL(5, 101, 22), // "viewPushButton_clicked"
QT_MOC_LITERAL(6, 124, 23), // "folioPushButton_clicked"
QT_MOC_LITERAL(7, 148, 25), // "archivePushButton_clicked"
QT_MOC_LITERAL(8, 174, 16), // "buttonOk_clicked"
QT_MOC_LITERAL(9, 191, 20), // "resolveLocalHostname"
QT_MOC_LITERAL(10, 212, 14) // "languageChange"

    },
    "RecordDialog\0deltaUnitsComboBox_activated\0"
    "\0selectedRadioButton_clicked\0"
    "allGadgetsRadioButton_clicked\0"
    "viewPushButton_clicked\0folioPushButton_clicked\0"
    "archivePushButton_clicked\0buttonOk_clicked\0"
    "resolveLocalHostname\0languageChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RecordDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       3,    0,   62,    2, 0x0a /* Public */,
       4,    0,   63,    2, 0x0a /* Public */,
       5,    0,   64,    2, 0x0a /* Public */,
       6,    0,   65,    2, 0x0a /* Public */,
       7,    0,   66,    2, 0x0a /* Public */,
       8,    0,   67,    2, 0x0a /* Public */,
       9,    1,   68,    2, 0x0a /* Public */,
      10,    0,   71,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString, QMetaType::QString,    2,
    QMetaType::Void,

       0        // eod
};

void RecordDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RecordDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->deltaUnitsComboBox_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->selectedRadioButton_clicked(); break;
        case 2: _t->allGadgetsRadioButton_clicked(); break;
        case 3: _t->viewPushButton_clicked(); break;
        case 4: _t->folioPushButton_clicked(); break;
        case 5: _t->archivePushButton_clicked(); break;
        case 6: _t->buttonOk_clicked(); break;
        case 7: { QString _r = _t->resolveLocalHostname((*reinterpret_cast< QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->languageChange(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RecordDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_RecordDialog.data,
    qt_meta_data_RecordDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RecordDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RecordDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RecordDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::RecordDialog"))
        return static_cast< Ui::RecordDialog*>(this);
    return QDialog::qt_metacast(_clname);
}

int RecordDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
struct qt_meta_stringdata_PmLogger_t {
    QByteArrayData data[5];
    char stringdata0[50];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PmLogger_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PmLogger_t qt_meta_stringdata_PmLogger = {
    {
QT_MOC_LITERAL(0, 0, 8), // "PmLogger"
QT_MOC_LITERAL(1, 9, 9), // "terminate"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 8), // "finished"
QT_MOC_LITERAL(4, 29, 20) // "QProcess::ExitStatus"

    },
    "PmLogger\0terminate\0\0finished\0"
    "QProcess::ExitStatus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PmLogger[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    2,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 4,    2,    2,

       0        // eod
};

void PmLogger::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PmLogger *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->terminate(); break;
        case 1: _t->finished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PmLogger::staticMetaObject = { {
    QMetaObject::SuperData::link<QProcess::staticMetaObject>(),
    qt_meta_stringdata_PmLogger.data,
    qt_meta_data_PmLogger,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PmLogger::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PmLogger::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PmLogger.stringdata0))
        return static_cast<void*>(this);
    return QProcess::qt_metacast(_clname);
}

int PmLogger::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QProcess::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_RecordFileDialog_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RecordFileDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RecordFileDialog_t qt_meta_stringdata_RecordFileDialog = {
    {
QT_MOC_LITERAL(0, 0, 16) // "RecordFileDialog"

    },
    "RecordFileDialog"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RecordFileDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void RecordFileDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject RecordFileDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QFileDialog::staticMetaObject>(),
    qt_meta_stringdata_RecordFileDialog.data,
    qt_meta_data_RecordFileDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RecordFileDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RecordFileDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RecordFileDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::RecordDialog"))
        return static_cast< Ui::RecordDialog*>(this);
    return QFileDialog::qt_metacast(_clname);
}

int RecordFileDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFileDialog::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
