/********************************************************************************
** Form generated from reading UI file 'samplesdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SAMPLESDIALOG_H
#define UI_SAMPLESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>

QT_BEGIN_NAMESPACE

class Ui_SamplesDialog
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *sampleTextLabel;
    QSpacerItem *spacerItem;
    QSpinBox *sampleCounter;
    QSpacerItem *spacerItem1;
    QSlider *visibleSlider;
    QSlider *sampleSlider;
    QHBoxLayout *hboxLayout1;
    QLabel *visibleTextLabel;
    QSpacerItem *spacerItem2;
    QSpinBox *visibleCounter;
    QSpacerItem *spacerItem3;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacerItem4;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QDialog *SamplesDialog)
    {
        if (SamplesDialog->objectName().isEmpty())
            SamplesDialog->setObjectName(QString::fromUtf8("SamplesDialog"));
        SamplesDialog->setWindowModality(Qt::WindowModal);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SamplesDialog->sizePolicy().hasHeightForWidth());
        SamplesDialog->setSizePolicy(sizePolicy);
        SamplesDialog->setMinimumSize(QSize(330, 220));
        SamplesDialog->setMaximumSize(QSize(330, 220));
        SamplesDialog->setFocusPolicy(Qt::WheelFocus);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/zoom-out.png"));
        SamplesDialog->setWindowIcon(icon);
        SamplesDialog->setSizeGripEnabled(false);
        gridLayout = new QGridLayout(SamplesDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout1 = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(0);
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        sampleTextLabel = new QLabel(SamplesDialog);
        sampleTextLabel->setObjectName(QString::fromUtf8("sampleTextLabel"));
        sampleTextLabel->setWordWrap(false);

        hboxLayout->addWidget(sampleTextLabel);

        spacerItem = new QSpacerItem(26, 20, QSizePolicy::Preferred, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        sampleCounter = new QSpinBox(SamplesDialog);
        sampleCounter->setObjectName(QString::fromUtf8("sampleCounter"));
        sampleCounter->setMaximum(999999999);

        hboxLayout->addWidget(sampleCounter);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);


        gridLayout1->addLayout(hboxLayout, 2, 0, 1, 1);

        visibleSlider = new QSlider(SamplesDialog);
        visibleSlider->setObjectName(QString::fromUtf8("visibleSlider"));
        visibleSlider->setOrientation(Qt::Horizontal);

        gridLayout1->addWidget(visibleSlider, 1, 0, 1, 1);

        sampleSlider = new QSlider(SamplesDialog);
        sampleSlider->setObjectName(QString::fromUtf8("sampleSlider"));
        sampleSlider->setOrientation(Qt::Horizontal);

        gridLayout1->addWidget(sampleSlider, 3, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(0);
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        visibleTextLabel = new QLabel(SamplesDialog);
        visibleTextLabel->setObjectName(QString::fromUtf8("visibleTextLabel"));
        visibleTextLabel->setWordWrap(false);

        hboxLayout1->addWidget(visibleTextLabel);

        spacerItem2 = new QSpacerItem(26, 20, QSizePolicy::Preferred, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem2);

        visibleCounter = new QSpinBox(SamplesDialog);
        visibleCounter->setObjectName(QString::fromUtf8("visibleCounter"));
        visibleCounter->setMaximum(999999999);

        hboxLayout1->addWidget(visibleCounter);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem3);


        gridLayout1->addLayout(hboxLayout1, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 5, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(verticalSpacer, 1, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        spacerItem4 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem4);

        buttonOk = new QPushButton(SamplesDialog);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout2->addWidget(buttonOk);

        buttonCancel = new QPushButton(SamplesDialog);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout2->addWidget(buttonCancel);


        gridLayout->addLayout(hboxLayout2, 2, 0, 1, 1);


        retranslateUi(SamplesDialog);

        buttonOk->setDefault(true);
        buttonCancel->setDefault(false);


        QMetaObject::connectSlotsByName(SamplesDialog);
    } // setupUi

    void retranslateUi(QDialog *SamplesDialog)
    {
        SamplesDialog->setWindowTitle(QCoreApplication::translate("SamplesDialog", "Add Samples", nullptr));
        sampleTextLabel->setText(QCoreApplication::translate("SamplesDialog", "Total Samples:", nullptr));
        visibleTextLabel->setText(QCoreApplication::translate("SamplesDialog", "Visible Points:", nullptr));
        buttonOk->setText(QCoreApplication::translate("SamplesDialog", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        buttonCancel->setText(QCoreApplication::translate("SamplesDialog", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class SamplesDialog: public Ui_SamplesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SAMPLESDIALOG_H
