/****************************************************************************
** Meta object code from reading C++ file 'pmchart.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "pmchart.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pmchart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PmChart_t {
    QByteArrayData data[48];
    char stringdata0[582];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PmChart_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PmChart_t qt_meta_stringdata_PmChart = {
    {
QT_MOC_LITERAL(0, 0, 7), // "PmChart"
QT_MOC_LITERAL(1, 8, 4), // "init"
QT_MOC_LITERAL(2, 13, 0), // ""
QT_MOC_LITERAL(3, 14, 4), // "quit"
QT_MOC_LITERAL(4, 19, 8), // "enableUi"
QT_MOC_LITERAL(5, 28, 10), // "exportFile"
QT_MOC_LITERAL(6, 39, 12), // "setupDialogs"
QT_MOC_LITERAL(7, 52, 12), // "fileOpenView"
QT_MOC_LITERAL(8, 65, 12), // "fileSaveView"
QT_MOC_LITERAL(9, 78, 10), // "fileExport"
QT_MOC_LITERAL(10, 89, 12), // "acceptExport"
QT_MOC_LITERAL(11, 102, 9), // "filePrint"
QT_MOC_LITERAL(12, 112, 8), // "fileQuit"
QT_MOC_LITERAL(13, 121, 10), // "helpManual"
QT_MOC_LITERAL(14, 132, 9), // "helpAbout"
QT_MOC_LITERAL(15, 142, 11), // "helpAboutQt"
QT_MOC_LITERAL(16, 154, 11), // "helpSeeAlso"
QT_MOC_LITERAL(17, 166, 9), // "whatsThis"
QT_MOC_LITERAL(18, 176, 22), // "optionsShowTimeControl"
QT_MOC_LITERAL(19, 199, 22), // "optionsHideTimeControl"
QT_MOC_LITERAL(20, 222, 14), // "optionsToolbar"
QT_MOC_LITERAL(21, 237, 14), // "optionsConsole"
QT_MOC_LITERAL(22, 252, 17), // "optionsNewPmchart"
QT_MOC_LITERAL(23, 270, 12), // "fileNewChart"
QT_MOC_LITERAL(24, 283, 9), // "editChart"
QT_MOC_LITERAL(25, 293, 15), // "acceptEditChart"
QT_MOC_LITERAL(26, 309, 10), // "closeChart"
QT_MOC_LITERAL(27, 320, 7), // "editTab"
QT_MOC_LITERAL(28, 328, 13), // "acceptEditTab"
QT_MOC_LITERAL(29, 342, 11), // "editSamples"
QT_MOC_LITERAL(30, 354, 17), // "acceptEditSamples"
QT_MOC_LITERAL(31, 372, 6), // "addTab"
QT_MOC_LITERAL(32, 379, 12), // "acceptNewTab"
QT_MOC_LITERAL(33, 392, 8), // "closeTab"
QT_MOC_LITERAL(34, 401, 16), // "activeTabChanged"
QT_MOC_LITERAL(35, 418, 12), // "editSettings"
QT_MOC_LITERAL(36, 431, 11), // "recordStart"
QT_MOC_LITERAL(37, 443, 11), // "recordQuery"
QT_MOC_LITERAL(38, 455, 10), // "recordStop"
QT_MOC_LITERAL(39, 466, 12), // "recordDetach"
QT_MOC_LITERAL(40, 479, 7), // "timeout"
QT_MOC_LITERAL(41, 487, 6), // "zoomIn"
QT_MOC_LITERAL(42, 494, 7), // "zoomOut"
QT_MOC_LITERAL(43, 502, 24), // "updateToolbarOrientation"
QT_MOC_LITERAL(44, 527, 15), // "Qt::Orientation"
QT_MOC_LITERAL(45, 543, 14), // "languageChange"
QT_MOC_LITERAL(46, 558, 10), // "closeEvent"
QT_MOC_LITERAL(47, 569, 12) // "QCloseEvent*"

    },
    "PmChart\0init\0\0quit\0enableUi\0exportFile\0"
    "setupDialogs\0fileOpenView\0fileSaveView\0"
    "fileExport\0acceptExport\0filePrint\0"
    "fileQuit\0helpManual\0helpAbout\0helpAboutQt\0"
    "helpSeeAlso\0whatsThis\0optionsShowTimeControl\0"
    "optionsHideTimeControl\0optionsToolbar\0"
    "optionsConsole\0optionsNewPmchart\0"
    "fileNewChart\0editChart\0acceptEditChart\0"
    "closeChart\0editTab\0acceptEditTab\0"
    "editSamples\0acceptEditSamples\0addTab\0"
    "acceptNewTab\0closeTab\0activeTabChanged\0"
    "editSettings\0recordStart\0recordQuery\0"
    "recordStop\0recordDetach\0timeout\0zoomIn\0"
    "zoomOut\0updateToolbarOrientation\0"
    "Qt::Orientation\0languageChange\0"
    "closeEvent\0QCloseEvent*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PmChart[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      44,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  234,    2, 0x0a /* Public */,
       3,    0,  235,    2, 0x0a /* Public */,
       4,    0,  236,    2, 0x0a /* Public */,
       5,    0,  237,    2, 0x0a /* Public */,
       6,    0,  238,    2, 0x0a /* Public */,
       7,    0,  239,    2, 0x0a /* Public */,
       8,    0,  240,    2, 0x0a /* Public */,
       9,    0,  241,    2, 0x0a /* Public */,
      10,    0,  242,    2, 0x0a /* Public */,
      11,    0,  243,    2, 0x0a /* Public */,
      12,    0,  244,    2, 0x0a /* Public */,
      13,    0,  245,    2, 0x0a /* Public */,
      14,    0,  246,    2, 0x0a /* Public */,
      15,    0,  247,    2, 0x0a /* Public */,
      16,    0,  248,    2, 0x0a /* Public */,
      17,    0,  249,    2, 0x0a /* Public */,
      18,    0,  250,    2, 0x0a /* Public */,
      19,    0,  251,    2, 0x0a /* Public */,
      20,    0,  252,    2, 0x0a /* Public */,
      21,    0,  253,    2, 0x0a /* Public */,
      22,    0,  254,    2, 0x0a /* Public */,
      23,    0,  255,    2, 0x0a /* Public */,
      24,    0,  256,    2, 0x0a /* Public */,
      25,    0,  257,    2, 0x0a /* Public */,
      26,    0,  258,    2, 0x0a /* Public */,
      27,    0,  259,    2, 0x0a /* Public */,
      28,    0,  260,    2, 0x0a /* Public */,
      29,    0,  261,    2, 0x0a /* Public */,
      30,    0,  262,    2, 0x0a /* Public */,
      31,    0,  263,    2, 0x0a /* Public */,
      32,    0,  264,    2, 0x0a /* Public */,
      33,    0,  265,    2, 0x0a /* Public */,
      34,    1,  266,    2, 0x0a /* Public */,
      35,    0,  269,    2, 0x0a /* Public */,
      36,    0,  270,    2, 0x0a /* Public */,
      37,    0,  271,    2, 0x0a /* Public */,
      38,    0,  272,    2, 0x0a /* Public */,
      39,    0,  273,    2, 0x0a /* Public */,
      40,    0,  274,    2, 0x0a /* Public */,
      41,    0,  275,    2, 0x0a /* Public */,
      42,    0,  276,    2, 0x0a /* Public */,
      43,    1,  277,    2, 0x0a /* Public */,
      45,    0,  280,    2, 0x09 /* Protected */,
      46,    1,  281,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 44,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 47,    2,

       0        // eod
};

void PmChart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PmChart *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->init(); break;
        case 1: _t->quit(); break;
        case 2: _t->enableUi(); break;
        case 3: _t->exportFile(); break;
        case 4: _t->setupDialogs(); break;
        case 5: _t->fileOpenView(); break;
        case 6: _t->fileSaveView(); break;
        case 7: _t->fileExport(); break;
        case 8: _t->acceptExport(); break;
        case 9: _t->filePrint(); break;
        case 10: _t->fileQuit(); break;
        case 11: _t->helpManual(); break;
        case 12: _t->helpAbout(); break;
        case 13: _t->helpAboutQt(); break;
        case 14: _t->helpSeeAlso(); break;
        case 15: _t->whatsThis(); break;
        case 16: _t->optionsShowTimeControl(); break;
        case 17: _t->optionsHideTimeControl(); break;
        case 18: _t->optionsToolbar(); break;
        case 19: _t->optionsConsole(); break;
        case 20: _t->optionsNewPmchart(); break;
        case 21: _t->fileNewChart(); break;
        case 22: _t->editChart(); break;
        case 23: _t->acceptEditChart(); break;
        case 24: _t->closeChart(); break;
        case 25: _t->editTab(); break;
        case 26: _t->acceptEditTab(); break;
        case 27: _t->editSamples(); break;
        case 28: _t->acceptEditSamples(); break;
        case 29: _t->addTab(); break;
        case 30: _t->acceptNewTab(); break;
        case 31: _t->closeTab(); break;
        case 32: _t->activeTabChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->editSettings(); break;
        case 34: _t->recordStart(); break;
        case 35: _t->recordQuery(); break;
        case 36: _t->recordStop(); break;
        case 37: _t->recordDetach(); break;
        case 38: _t->timeout(); break;
        case 39: _t->zoomIn(); break;
        case 40: _t->zoomOut(); break;
        case 41: _t->updateToolbarOrientation((*reinterpret_cast< Qt::Orientation(*)>(_a[1]))); break;
        case 42: _t->languageChange(); break;
        case 43: _t->closeEvent((*reinterpret_cast< QCloseEvent*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PmChart::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_PmChart.data,
    qt_meta_data_PmChart,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PmChart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PmChart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PmChart.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PmChart"))
        return static_cast< Ui::PmChart*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int PmChart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 44;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
