/*
 * This file was generated automatically by ExtUtils::ParseXS version 3.57 from the
 * contents of LogImport.xs. Do not edit this file, edit LogImport.xs instead.
 *
 *    ANY CHANGES MADE HERE WILL BE LOST!
 *
 */

#line 1 "LogImport.xs"
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#include <pmapi.h>
#include <libpcp.h>
#include <import.h>

#line 18 "LogImport.c"
#ifndef PERL_UNUSED_VAR
#  define PERL_UNUSED_VAR(var) if (0) var = var
#endif

#ifndef dVAR
#  define dVAR		dNOOP
#endif


/* This stuff is not part of the API! You have been warned. */
#ifndef PERL_VERSION_DECIMAL
#  define PERL_VERSION_DECIMAL(r,v,s) (r*1000000 + v*1000 + s)
#endif
#ifndef PERL_DECIMAL_VERSION
#  define PERL_DECIMAL_VERSION \
	  PERL_VERSION_DECIMAL(PERL_REVISION,PERL_VERSION,PERL_SUBVERSION)
#endif
#ifndef PERL_VERSION_GE
#  define PERL_VERSION_GE(r,v,s) \
	  (PERL_DECIMAL_VERSION >= PERL_VERSION_DECIMAL(r,v,s))
#endif
#ifndef PERL_VERSION_LE
#  define PERL_VERSION_LE(r,v,s) \
	  (PERL_DECIMAL_VERSION <= PERL_VERSION_DECIMAL(r,v,s))
#endif

/* XS_INTERNAL is the explicit static-linkage variant of the default
 * XS macro.
 *
 * XS_EXTERNAL is the same as XS_INTERNAL except it does not include
 * "STATIC", ie. it exports XSUB symbols. You probably don't want that
 * for anything but the BOOT XSUB.
 *
 * See XSUB.h in core!
 */


/* TODO: This might be compatible further back than 5.10.0. */
#if PERL_VERSION_GE(5, 10, 0) && PERL_VERSION_LE(5, 15, 1)
#  undef XS_EXTERNAL
#  undef XS_INTERNAL
#  if defined(__CYGWIN__) && defined(USE_DYNAMIC_LOADING)
#    define XS_EXTERNAL(name) __declspec(dllexport) XSPROTO(name)
#    define XS_INTERNAL(name) STATIC XSPROTO(name)
#  endif
#  if defined(__SYMBIAN32__)
#    define XS_EXTERNAL(name) EXPORT_C XSPROTO(name)
#    define XS_INTERNAL(name) EXPORT_C STATIC XSPROTO(name)
#  endif
#  ifndef XS_EXTERNAL
#    if defined(HASATTRIBUTE_UNUSED) && !defined(__cplusplus)
#      define XS_EXTERNAL(name) void name(pTHX_ CV* cv __attribute__unused__)
#      define XS_INTERNAL(name) STATIC void name(pTHX_ CV* cv __attribute__unused__)
#    else
#      ifdef __cplusplus
#        define XS_EXTERNAL(name) extern "C" XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      else
#        define XS_EXTERNAL(name) XSPROTO(name)
#        define XS_INTERNAL(name) STATIC XSPROTO(name)
#      endif
#    endif
#  endif
#endif

/* perl >= 5.10.0 && perl <= 5.15.1 */


/* The XS_EXTERNAL macro is used for functions that must not be static
 * like the boot XSUB of a module. If perl didn't have an XS_EXTERNAL
 * macro defined, the best we can do is assume XS is the same.
 * Dito for XS_INTERNAL.
 */
#ifndef XS_EXTERNAL
#  define XS_EXTERNAL(name) XS(name)
#endif
#ifndef XS_INTERNAL
#  define XS_INTERNAL(name) XS(name)
#endif

/* Now, finally, after all this mess, we want an ExtUtils::ParseXS
 * internal macro that we're free to redefine for varying linkage due
 * to the EXPORT_XSUB_SYMBOLS XS keyword. This is internal, use
 * XS_EXTERNAL(name) or XS_INTERNAL(name) in your code if you need to!
 */

#undef XS_EUPXS
#if defined(PERL_EUPXS_ALWAYS_EXPORT)
#  define XS_EUPXS(name) XS_EXTERNAL(name)
#else
   /* default to internal */
#  define XS_EUPXS(name) XS_INTERNAL(name)
#endif

#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)

/* prototype to pass -Wmissing-prototypes */
STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params);

STATIC void
S_croak_xs_usage(const CV *const cv, const char *const params)
{
    const GV *const gv = CvGV(cv);

    PERL_ARGS_ASSERT_CROAK_XS_USAGE;

    if (gv) {
        const char *const gvname = GvNAME(gv);
        const HV *const stash = GvSTASH(gv);
        const char *const hvname = stash ? HvNAME(stash) : NULL;

        if (hvname)
	    Perl_croak_nocontext("Usage: %s::%s(%s)", hvname, gvname, params);
        else
	    Perl_croak_nocontext("Usage: %s(%s)", gvname, params);
    } else {
        /* Pants. I don't think that it should be possible to get here. */
	Perl_croak_nocontext("Usage: CODE(0x%" UVxf ")(%s)", PTR2UV(cv), params);
    }
}
#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE

#define croak_xs_usage        S_croak_xs_usage

#endif

/* NOTE: the prototype of newXSproto() is different in versions of perls,
 * so we define a portable version of newXSproto()
 */
#ifdef newXS_flags
#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
#else
#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
#endif /* !defined(newXS_flags) */

#if PERL_VERSION_LE(5, 21, 5)
#  define newXS_deffile(a,b) Perl_newXS(aTHX_ a,b,file)
#else
#  define newXS_deffile(a,b) Perl_newXS_deffile(aTHX_ a,b)
#endif

/* simple backcompat versions of the TARGx() macros with no optimisation */
#ifndef TARGi
#  define TARGi(iv, do_taint) sv_setiv_mg(TARG, iv)
#  define TARGu(uv, do_taint) sv_setuv_mg(TARG, uv)
#  define TARGn(nv, do_taint) sv_setnv_mg(TARG, nv)
#endif

#line 169 "LogImport.c"

XS_EUPXS(XS_PCP__LogImport_pmID_build); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmID_build)
{
    dVAR; dXSARGS;
    if (items != 3)
       croak_xs_usage(cv,  "domain, cluster, item");
    {
	unsigned int	domain = (unsigned int)SvUV(ST(0))
;
	unsigned int	cluster = (unsigned int)SvUV(ST(1))
;
	unsigned int	item = (unsigned int)SvUV(ST(2))
;
	pmID	RETVAL;
	dXSTARG;
#line 21 "LogImport.xs"
	RETVAL = pmID_build(domain, cluster, item);
#line 188 "LogImport.c"
	TARGi(*((int *)&RETVAL), 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmid_build); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmid_build)
{
    dVAR; dXSARGS;
    if (items != 3)
       croak_xs_usage(cv,  "domain, cluster, item");
    {
	unsigned int	domain = (unsigned int)SvUV(ST(0))
;
	unsigned int	cluster = (unsigned int)SvUV(ST(1))
;
	unsigned int	item = (unsigned int)SvUV(ST(2))
;
	pmID	RETVAL;
	dXSTARG;
#line 32 "LogImport.xs"
	RETVAL = pmID_build(domain, cluster, item);
#line 213 "LogImport.c"
	TARGi(*((int *)&RETVAL), 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmInDom_build); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmInDom_build)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "domain, serial");
    {
	unsigned int	domain = (unsigned int)SvUV(ST(0))
;
	unsigned int	serial = (unsigned int)SvUV(ST(1))
;
	pmInDom	RETVAL;
	dXSTARG;
#line 43 "LogImport.xs"
	RETVAL = pmInDom_build(domain, serial);
#line 236 "LogImport.c"
	TARGi(*((int *)&RETVAL), 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmSetDebug); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmSetDebug)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "options");
    {
	char *	options = (char *)SvPV_nolen(ST(0))
;
	int	RETVAL;
	dXSTARG;
#line 53 "LogImport.xs"
	RETVAL = pmSetDebug(options);
#line 257 "LogImport.c"
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiDump); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiDump)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {

	pmiDump();
    }
    XSRETURN_EMPTY;
}


XS_EUPXS(XS_PCP__LogImport_pmiUnits); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiUnits)
{
    dVAR; dXSARGS;
    if (items != 6)
       croak_xs_usage(cv,  "dimSpace, dimTime, dimCount, scaleSpace, scaleTime, scaleCount");
    {
	int	dimSpace = (int)SvIV(ST(0))
;
	int	dimTime = (int)SvIV(ST(1))
;
	int	dimCount = (int)SvIV(ST(2))
;
	int	scaleSpace = (int)SvIV(ST(3))
;
	int	scaleTime = (int)SvIV(ST(4))
;
	int	scaleCount = (int)SvIV(ST(5))
;
	pmUnits	RETVAL;
	dXSTARG;

	RETVAL = pmiUnits(dimSpace, dimTime, dimCount, scaleSpace, scaleTime, scaleCount);
	TARGi(*((int *)&RETVAL), 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiID); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiID)
{
    dVAR; dXSARGS;
    if (items != 3)
       croak_xs_usage(cv,  "domain, cluster, item");
    {
	int	domain = (int)SvIV(ST(0))
;
	int	cluster = (int)SvIV(ST(1))
;
	int	item = (int)SvIV(ST(2))
;
	pmID	RETVAL;
	dXSTARG;

	RETVAL = pmiID(domain, cluster, item);
	TARGi(*((int *)&RETVAL), 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiInDom); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiInDom)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "domain, serial");
    {
	int	domain = (int)SvIV(ST(0))
;
	int	serial = (int)SvIV(ST(1))
;
	pmInDom	RETVAL;
	dXSTARG;

	RETVAL = pmiInDom(domain, serial);
	TARGi(*((int *)&RETVAL), 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiErrStr); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiErrStr)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "sts");
    {
	int	sts = (int)SvIV(ST(0))
;
	const char *	RETVAL;
	dXSTARG;

	RETVAL = pmiErrStr(sts);
	sv_setpv((SV*)TARG, RETVAL);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiStart); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiStart)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "archive, inherit");
    {
	char *	archive = (char *)SvPV_nolen(ST(0))
;
	int	inherit = (int)SvIV(ST(1))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiStart(archive, inherit);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiUseContext); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiUseContext)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "context");
    {
	int	context = (int)SvIV(ST(0))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiUseContext(context);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiEnd); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiEnd)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiEnd();
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiSetHostname); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiSetHostname)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "value");
    {
	char *	value = (char *)SvPV_nolen(ST(0))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiSetHostname(value);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiSetTimezone); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiSetTimezone)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "value");
    {
	char *	value = (char *)SvPV_nolen(ST(0))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiSetTimezone(value);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiSetVersion); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiSetVersion)
{
    dVAR; dXSARGS;
    if (items != 1)
       croak_xs_usage(cv,  "value");
    {
	int	value = (int)SvIV(ST(0))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiSetVersion(value);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiAddMetric); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiAddMetric)
{
    dVAR; dXSARGS;
    if (items != 6)
       croak_xs_usage(cv,  "name, pmid, type, indom, sem, units");
    {
	const char *	name = (const char *)SvPV_nolen(ST(0))
;
	pmID	pmid;
	int	type = (int)SvIV(ST(2))
;
	pmInDom	indom;
	int	sem = (int)SvIV(ST(4))
;
	pmUnits	units;
	int	RETVAL;
	dXSTARG;

	{
	    __int32_t tmp = SvIV(ST(1));
	    memcpy((void *)&pmid, (void *)&tmp, sizeof(__int32_t));
	}
;

	{
	    __int32_t tmp = SvIV(ST(3));
	    memcpy((void *)&indom, (void *)&tmp, sizeof(__int32_t));
	}
;

	{
	    __int32_t tmp = SvIV(ST(5));
	    memcpy((void *)&units, (void *)&tmp, sizeof(__int32_t));
	}
;

	RETVAL = pmiAddMetric(name, pmid, type, indom, sem, units);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiAddInstance); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiAddInstance)
{
    dVAR; dXSARGS;
    if (items != 3)
       croak_xs_usage(cv,  "indom, instance, inst");
    {
	pmInDom	indom;
	const char *	instance = (const char *)SvPV_nolen(ST(1))
;
	int	inst = (int)SvIV(ST(2))
;
	int	RETVAL;
	dXSTARG;

	{
	    __int32_t tmp = SvIV(ST(0));
	    memcpy((void *)&indom, (void *)&tmp, sizeof(__int32_t));
	}
;

	RETVAL = pmiAddInstance(indom, instance, inst);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiPutValue); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiPutValue)
{
    dVAR; dXSARGS;
    if (items != 3)
       croak_xs_usage(cv,  "name, instance, value");
    {
	const char *	name = (const char *)SvPV_nolen(ST(0))
;
	const char *	instance = (const char *)SvPV_nolen(ST(1))
;
	const char *	value = (const char *)SvPV_nolen(ST(2))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiPutValue(name, instance, value);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiGetHandle); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiGetHandle)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "name, instance");
    {
	const char *	name = (const char *)SvPV_nolen(ST(0))
;
	const char *	instance = (const char *)SvPV_nolen(ST(1))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiGetHandle(name, instance);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiPutValueHandle); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiPutValueHandle)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "handle, value");
    {
	int	handle = (int)SvIV(ST(0))
;
	const char *	value = (const char *)SvPV_nolen(ST(1))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiPutValueHandle(handle, value);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiPutText); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiPutText)
{
    dVAR; dXSARGS;
    if (items != 4)
       croak_xs_usage(cv,  "type, class, id, content");
    {
	unsigned int	type = (unsigned int)SvUV(ST(0))
;
	unsigned int	class = (unsigned int)SvUV(ST(1))
;
	unsigned int	id = (unsigned int)SvUV(ST(2))
;
	const char *	content = (const char *)SvPV_nolen(ST(3))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiPutText(type, class, id, content);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiPutLabel); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiPutLabel)
{
    dVAR; dXSARGS;
    if (items != 5)
       croak_xs_usage(cv,  "type, id, inst, name, content");
    {
	unsigned int	type = (unsigned int)SvUV(ST(0))
;
	unsigned int	id = (unsigned int)SvUV(ST(1))
;
	unsigned int	inst = (unsigned int)SvUV(ST(2))
;
	const char *	name = (const char *)SvPV_nolen(ST(3))
;
	const char *	content = (const char *)SvPV_nolen(ST(4))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiPutLabel(type, id, inst, name, content);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiWrite); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiWrite)
{
    dVAR; dXSARGS;
    if (items != 2)
       croak_xs_usage(cv,  "sec, usec");
    {
	int	sec = (int)SvIV(ST(0))
;
	int	usec = (int)SvIV(ST(1))
;
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiWrite(sec, usec);
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}


XS_EUPXS(XS_PCP__LogImport_pmiPutMark); /* prototype to pass -Wmissing-prototypes */
XS_EUPXS(XS_PCP__LogImport_pmiPutMark)
{
    dVAR; dXSARGS;
    if (items != 0)
       croak_xs_usage(cv,  "");
    {
	int	RETVAL;
	dXSTARG;

	RETVAL = pmiPutMark();
	TARGi((IV)RETVAL, 1);
	ST(0) = TARG;
    }
    XSRETURN(1);
}

#ifdef __cplusplus
extern "C" {
#endif
XS_EXTERNAL(boot_PCP__LogImport); /* prototype to pass -Wmissing-prototypes */
XS_EXTERNAL(boot_PCP__LogImport)
{
#if PERL_VERSION_LE(5, 21, 5)
    dVAR; dXSARGS;
#else
    dVAR; dXSBOOTARGSXSAPIVERCHK;
#endif
#if PERL_VERSION_LE(5, 8, 999) /* PERL_VERSION_LT is 5.33+ */
    char* file = __FILE__;
#else
    const char* file = __FILE__;
#endif

    PERL_UNUSED_VAR(file);

    PERL_UNUSED_VAR(cv); /* -W */
    PERL_UNUSED_VAR(items); /* -W */
#if PERL_VERSION_LE(5, 21, 5)
    XS_VERSION_BOOTCHECK;
#  ifdef XS_APIVERSION_BOOTCHECK
    XS_APIVERSION_BOOTCHECK;
#  endif
#endif

        newXS_deffile("PCP::LogImport::pmID_build", XS_PCP__LogImport_pmID_build);
        newXS_deffile("PCP::LogImport::pmid_build", XS_PCP__LogImport_pmid_build);
        newXS_deffile("PCP::LogImport::pmInDom_build", XS_PCP__LogImport_pmInDom_build);
        newXS_deffile("PCP::LogImport::pmSetDebug", XS_PCP__LogImport_pmSetDebug);
        newXS_deffile("PCP::LogImport::pmiDump", XS_PCP__LogImport_pmiDump);
        newXS_deffile("PCP::LogImport::pmiUnits", XS_PCP__LogImport_pmiUnits);
        newXS_deffile("PCP::LogImport::pmiID", XS_PCP__LogImport_pmiID);
        newXS_deffile("PCP::LogImport::pmiInDom", XS_PCP__LogImport_pmiInDom);
        newXS_deffile("PCP::LogImport::pmiErrStr", XS_PCP__LogImport_pmiErrStr);
        newXS_deffile("PCP::LogImport::pmiStart", XS_PCP__LogImport_pmiStart);
        newXS_deffile("PCP::LogImport::pmiUseContext", XS_PCP__LogImport_pmiUseContext);
        newXS_deffile("PCP::LogImport::pmiEnd", XS_PCP__LogImport_pmiEnd);
        newXS_deffile("PCP::LogImport::pmiSetHostname", XS_PCP__LogImport_pmiSetHostname);
        newXS_deffile("PCP::LogImport::pmiSetTimezone", XS_PCP__LogImport_pmiSetTimezone);
        newXS_deffile("PCP::LogImport::pmiSetVersion", XS_PCP__LogImport_pmiSetVersion);
        newXS_deffile("PCP::LogImport::pmiAddMetric", XS_PCP__LogImport_pmiAddMetric);
        newXS_deffile("PCP::LogImport::pmiAddInstance", XS_PCP__LogImport_pmiAddInstance);
        newXS_deffile("PCP::LogImport::pmiPutValue", XS_PCP__LogImport_pmiPutValue);
        newXS_deffile("PCP::LogImport::pmiGetHandle", XS_PCP__LogImport_pmiGetHandle);
        newXS_deffile("PCP::LogImport::pmiPutValueHandle", XS_PCP__LogImport_pmiPutValueHandle);
        newXS_deffile("PCP::LogImport::pmiPutText", XS_PCP__LogImport_pmiPutText);
        newXS_deffile("PCP::LogImport::pmiPutLabel", XS_PCP__LogImport_pmiPutLabel);
        newXS_deffile("PCP::LogImport::pmiWrite", XS_PCP__LogImport_pmiWrite);
        newXS_deffile("PCP::LogImport::pmiPutMark", XS_PCP__LogImport_pmiPutMark);
#if PERL_VERSION_LE(5, 21, 5)
#  if PERL_VERSION_GE(5, 9, 0)
    if (PL_unitcheckav)
        call_list(PL_scopestack_ix, PL_unitcheckav);
#  endif
    XSRETURN_YES;
#else
    Perl_xs_boot_epilog(aTHX_ ax);
#endif
}

#ifdef __cplusplus
}
#endif
