/****************************************************************************
** Meta object code from reading C++ file 'openviewdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "openviewdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'openviewdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OpenViewDialog_t {
    QByteArrayData data[12];
    char stringdata0[268];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OpenViewDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OpenViewDialog_t qt_meta_stringdata_OpenViewDialog = {
    {
QT_MOC_LITERAL(0, 0, 14), // "OpenViewDialog"
QT_MOC_LITERAL(1, 15, 24), // "parentToolButton_clicked"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 22), // "userToolButton_clicked"
QT_MOC_LITERAL(4, 64, 24), // "systemToolButton_clicked"
QT_MOC_LITERAL(5, 89, 32), // "pathComboBox_currentIndexChanged"
QT_MOC_LITERAL(6, 122, 28), // "dirListView_selectionChanged"
QT_MOC_LITERAL(7, 151, 21), // "dirListView_activated"
QT_MOC_LITERAL(8, 173, 11), // "QModelIndex"
QT_MOC_LITERAL(9, 185, 34), // "sourceComboBox_currentIndexCh..."
QT_MOC_LITERAL(10, 220, 24), // "sourcePushButton_clicked"
QT_MOC_LITERAL(11, 245, 22) // "openPushButton_clicked"

    },
    "OpenViewDialog\0parentToolButton_clicked\0"
    "\0userToolButton_clicked\0"
    "systemToolButton_clicked\0"
    "pathComboBox_currentIndexChanged\0"
    "dirListView_selectionChanged\0"
    "dirListView_activated\0QModelIndex\0"
    "sourceComboBox_currentIndexChanged\0"
    "sourcePushButton_clicked\0"
    "openPushButton_clicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OpenViewDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x0a /* Public */,
       3,    1,   60,    2, 0x0a /* Public */,
       4,    1,   63,    2, 0x0a /* Public */,
       5,    1,   66,    2, 0x0a /* Public */,
       6,    0,   69,    2, 0x0a /* Public */,
       7,    1,   70,    2, 0x0a /* Public */,
       9,    1,   73,    2, 0x0a /* Public */,
      10,    0,   76,    2, 0x0a /* Public */,
      11,    0,   77,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void OpenViewDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OpenViewDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->parentToolButton_clicked(); break;
        case 1: _t->userToolButton_clicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->systemToolButton_clicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->pathComboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->dirListView_selectionChanged(); break;
        case 5: _t->dirListView_activated((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->sourceComboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->sourcePushButton_clicked(); break;
        case 8: _t->openPushButton_clicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OpenViewDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_OpenViewDialog.data,
    qt_meta_data_OpenViewDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OpenViewDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OpenViewDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OpenViewDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::OpenViewDialog"))
        return static_cast< Ui::OpenViewDialog*>(this);
    return QDialog::qt_metacast(_clname);
}

int OpenViewDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
