/********************************************************************************
** Form generated from reading UI file 'pmchart.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PMCHART_H
#define UI_PMCHART_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <tabwidget.h>
#include "chartdialog.h"
#include "exportdialog.h"
#include "infodialog.h"
#include "qed_timebutton.h"
#include "QtWidgets/QFileDialog"
#include "qmc_time.h"
#include "recorddialog.h"
#include "settingsdialog.h"
#include "tabdialog.h"
#include "timeaxis.h"

QT_BEGIN_NAMESPACE

class Ui_PmChart
{
public:
    QAction *fileOpenViewAction;
    QAction *fileSaveViewAction;
    QAction *fileExportAction;
    QAction *filePrintAction;
    QAction *fileQuitAction;
    QAction *newPmchartAction;
    QAction *showTimeControlAction;
    QAction *hideTimeControlAction;
    QAction *toolbarAction;
    QAction *consoleAction;
    QAction *editChartAction;
    QAction *editTabAction;
    QAction *editSamplesAction;
    QAction *editSettingsAction;
    QAction *closeTabAction;
    QAction *fileNewChartAction;
    QAction *addTabAction;
    QAction *closeChartAction;
    QAction *zoomInAction;
    QAction *zoomOutAction;
    QAction *helpManualAction;
    QAction *helpAboutAction;
    QAction *helpAboutQtAction;
    QAction *helpSeeAlsoAction;
    QAction *helpWhatsThisAction;
    QAction *recordStartAction;
    QAction *recordQueryAction;
    QAction *recordStopAction;
    QAction *recordDetachAction;
    QWidget *widget;
    QVBoxLayout *vboxLayout;
    QVBoxLayout *vboxLayout1;
    TabWidget *chartTabWidget;
    QHBoxLayout *hboxLayout;
    QMenuBar *MenuBar;
    QMenu *Help;
    QMenu *Time;
    QMenu *Record;
    QMenu *File;
    QMenu *Edit;
    QToolBar *toolBar;

    void setupUi(QMainWindow *PmChart)
    {
        if (PmChart->objectName().isEmpty())
            PmChart->setObjectName(QString::fromUtf8("PmChart"));
        PmChart->resize(370, 270);
        PmChart->setMinimumSize(QSize(182, 196));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/pmchart.png"), QSize(), QIcon::Normal, QIcon::Off);
        PmChart->setWindowIcon(icon);
        PmChart->setIconSize(QSize(22, 22));
        fileOpenViewAction = new QAction(PmChart);
        fileOpenViewAction->setObjectName(QString::fromUtf8("fileOpenViewAction"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/view.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileOpenViewAction->setIcon(icon1);
        fileSaveViewAction = new QAction(PmChart);
        fileSaveViewAction->setObjectName(QString::fromUtf8("fileSaveViewAction"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/document-save.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileSaveViewAction->setIcon(icon2);
        fileExportAction = new QAction(PmChart);
        fileExportAction->setObjectName(QString::fromUtf8("fileExportAction"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/images/document-export.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileExportAction->setIcon(icon3);
        filePrintAction = new QAction(PmChart);
        filePrintAction->setObjectName(QString::fromUtf8("filePrintAction"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/images/document-print.png"), QSize(), QIcon::Normal, QIcon::Off);
        filePrintAction->setIcon(icon4);
        fileQuitAction = new QAction(PmChart);
        fileQuitAction->setObjectName(QString::fromUtf8("fileQuitAction"));
        newPmchartAction = new QAction(PmChart);
        newPmchartAction->setObjectName(QString::fromUtf8("newPmchartAction"));
        newPmchartAction->setIcon(icon);
        showTimeControlAction = new QAction(PmChart);
        showTimeControlAction->setObjectName(QString::fromUtf8("showTimeControlAction"));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/images/pmtime.png"), QSize(), QIcon::Normal, QIcon::Off);
        showTimeControlAction->setIcon(icon5);
        hideTimeControlAction = new QAction(PmChart);
        hideTimeControlAction->setObjectName(QString::fromUtf8("hideTimeControlAction"));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/images/pmtime-close.png"), QSize(), QIcon::Normal, QIcon::Off);
        hideTimeControlAction->setIcon(icon6);
        toolbarAction = new QAction(PmChart);
        toolbarAction->setObjectName(QString::fromUtf8("toolbarAction"));
        toolbarAction->setCheckable(true);
        consoleAction = new QAction(PmChart);
        consoleAction->setObjectName(QString::fromUtf8("consoleAction"));
        consoleAction->setCheckable(true);
        editChartAction = new QAction(PmChart);
        editChartAction->setObjectName(QString::fromUtf8("editChartAction"));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/images/document-properties.png"), QSize(), QIcon::Normal, QIcon::Off);
        editChartAction->setIcon(icon7);
        editTabAction = new QAction(PmChart);
        editTabAction->setObjectName(QString::fromUtf8("editTabAction"));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/images/tab-edit.png"), QSize(), QIcon::Normal, QIcon::Off);
        editTabAction->setIcon(icon8);
        editSamplesAction = new QAction(PmChart);
        editSamplesAction->setObjectName(QString::fromUtf8("editSamplesAction"));
        editSettingsAction = new QAction(PmChart);
        editSettingsAction->setObjectName(QString::fromUtf8("editSettingsAction"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/images/settings.png"), QSize(), QIcon::Normal, QIcon::Off);
        editSettingsAction->setIcon(icon9);
        closeTabAction = new QAction(PmChart);
        closeTabAction->setObjectName(QString::fromUtf8("closeTabAction"));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/images/tab-close.png"), QSize(), QIcon::Normal, QIcon::Off);
        closeTabAction->setIcon(icon10);
        fileNewChartAction = new QAction(PmChart);
        fileNewChartAction->setObjectName(QString::fromUtf8("fileNewChartAction"));
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/images/document-new.png"), QSize(), QIcon::Normal, QIcon::Off);
        fileNewChartAction->setIcon(icon11);
        addTabAction = new QAction(PmChart);
        addTabAction->setObjectName(QString::fromUtf8("addTabAction"));
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/images/tab-new.png"), QSize(), QIcon::Normal, QIcon::Off);
        addTabAction->setIcon(icon12);
        closeChartAction = new QAction(PmChart);
        closeChartAction->setObjectName(QString::fromUtf8("closeChartAction"));
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/images/document-close.png"), QSize(), QIcon::Normal, QIcon::Off);
        closeChartAction->setIcon(icon13);
        zoomInAction = new QAction(PmChart);
        zoomInAction->setObjectName(QString::fromUtf8("zoomInAction"));
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/images/zoom-in.png"), QSize(), QIcon::Normal, QIcon::Off);
        zoomInAction->setIcon(icon14);
        zoomOutAction = new QAction(PmChart);
        zoomOutAction->setObjectName(QString::fromUtf8("zoomOutAction"));
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/images/zoom-out.png"), QSize(), QIcon::Normal, QIcon::Off);
        zoomOutAction->setIcon(icon15);
        helpManualAction = new QAction(PmChart);
        helpManualAction->setObjectName(QString::fromUtf8("helpManualAction"));
        QIcon icon16;
        icon16.addFile(QString::fromUtf8(":/images/help-browser.png"), QSize(), QIcon::Normal, QIcon::Off);
        helpManualAction->setIcon(icon16);
        helpAboutAction = new QAction(PmChart);
        helpAboutAction->setObjectName(QString::fromUtf8("helpAboutAction"));
        helpAboutAction->setIcon(icon);
        helpAboutQtAction = new QAction(PmChart);
        helpAboutQtAction->setObjectName(QString::fromUtf8("helpAboutQtAction"));
        helpSeeAlsoAction = new QAction(PmChart);
        helpSeeAlsoAction->setObjectName(QString::fromUtf8("helpSeeAlsoAction"));
        helpWhatsThisAction = new QAction(PmChart);
        helpWhatsThisAction->setObjectName(QString::fromUtf8("helpWhatsThisAction"));
        QIcon icon17;
        icon17.addFile(QString::fromUtf8(":/images/whatsthis.png"), QSize(), QIcon::Normal, QIcon::Off);
        helpWhatsThisAction->setIcon(icon17);
        recordStartAction = new QAction(PmChart);
        recordStartAction->setObjectName(QString::fromUtf8("recordStartAction"));
        QIcon icon18;
        icon18.addFile(QString::fromUtf8(":/images/camera-video.png"), QSize(), QIcon::Normal, QIcon::Off);
        recordStartAction->setIcon(icon18);
        recordQueryAction = new QAction(PmChart);
        recordQueryAction->setObjectName(QString::fromUtf8("recordQueryAction"));
        recordStopAction = new QAction(PmChart);
        recordStopAction->setObjectName(QString::fromUtf8("recordStopAction"));
        QIcon icon19;
        icon19.addFile(QString::fromUtf8(":/images/camera-video-close.png"), QSize(), QIcon::Normal, QIcon::Off);
        recordStopAction->setIcon(icon19);
        recordDetachAction = new QAction(PmChart);
        recordDetachAction->setObjectName(QString::fromUtf8("recordDetachAction"));
        widget = new QWidget(PmChart);
        widget->setObjectName(QString::fromUtf8("widget"));
        vboxLayout = new QVBoxLayout(widget);
        vboxLayout->setSpacing(0);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(0);
#ifndef Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        chartTabWidget = new TabWidget(widget);
        chartTabWidget->setObjectName(QString::fromUtf8("chartTabWidget"));
        chartTabWidget->setFocusPolicy(Qt::NoFocus);
        chartTabWidget->setTabShape(QTabWidget::Rounded);

        vboxLayout1->addWidget(chartTabWidget);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(2);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));

        vboxLayout1->addLayout(hboxLayout);


        vboxLayout->addLayout(vboxLayout1);

        PmChart->setCentralWidget(widget);
        MenuBar = new QMenuBar(PmChart);
        MenuBar->setObjectName(QString::fromUtf8("MenuBar"));
        MenuBar->setGeometry(QRect(0, 0, 370, 25));
        Help = new QMenu(MenuBar);
        Help->setObjectName(QString::fromUtf8("Help"));
        Time = new QMenu(MenuBar);
        Time->setObjectName(QString::fromUtf8("Time"));
        Record = new QMenu(MenuBar);
        Record->setObjectName(QString::fromUtf8("Record"));
        File = new QMenu(MenuBar);
        File->setObjectName(QString::fromUtf8("File"));
        Edit = new QMenu(MenuBar);
        Edit->setObjectName(QString::fromUtf8("Edit"));
        PmChart->setMenuBar(MenuBar);
        toolBar = new QToolBar(PmChart);
        toolBar->setObjectName(QString::fromUtf8("toolBar"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(toolBar->sizePolicy().hasHeightForWidth());
        toolBar->setSizePolicy(sizePolicy);
        toolBar->setMinimumSize(QSize(48, 36));
        toolBar->setOrientation(Qt::Horizontal);
        PmChart->addToolBar(Qt::TopToolBarArea, toolBar);

        MenuBar->addAction(File->menuAction());
        MenuBar->addAction(Edit->menuAction());
        MenuBar->addAction(Record->menuAction());
        MenuBar->addAction(Time->menuAction());
        MenuBar->addSeparator();
        MenuBar->addAction(Help->menuAction());
        Help->addAction(helpManualAction);
        Help->addSeparator();
        Help->addAction(helpAboutAction);
        Help->addAction(helpAboutQtAction);
        Help->addAction(helpSeeAlsoAction);
        Help->addSeparator();
        Help->addAction(helpWhatsThisAction);
        Time->addAction(showTimeControlAction);
        Time->addAction(hideTimeControlAction);
        Time->addAction(toolbarAction);
        Time->addAction(consoleAction);
        Time->addSeparator();
        Time->addAction(newPmchartAction);
        Record->addAction(recordStartAction);
        Record->addAction(recordQueryAction);
        Record->addAction(recordStopAction);
        Record->addSeparator();
        Record->addAction(recordDetachAction);
        File->addAction(fileNewChartAction);
        File->addAction(closeChartAction);
        File->addAction(fileOpenViewAction);
        File->addAction(addTabAction);
        File->addAction(closeTabAction);
        File->addSeparator();
        File->addAction(fileSaveViewAction);
        File->addSeparator();
        File->addAction(fileExportAction);
        File->addAction(filePrintAction);
        File->addSeparator();
        File->addAction(fileQuitAction);
        Edit->addAction(editChartAction);
        Edit->addAction(editTabAction);
        Edit->addSeparator();
        Edit->addAction(editSamplesAction);
        Edit->addAction(zoomInAction);
        Edit->addAction(zoomOutAction);
        Edit->addSeparator();
        Edit->addAction(editSettingsAction);
        toolBar->addAction(fileNewChartAction);
        toolBar->addAction(fileOpenViewAction);
        toolBar->addSeparator();
        toolBar->addAction(zoomInAction);
        toolBar->addAction(zoomOutAction);
        toolBar->addSeparator();
        toolBar->addAction(fileExportAction);
        toolBar->addSeparator();
        toolBar->addAction(showTimeControlAction);
        toolBar->addAction(newPmchartAction);

        retranslateUi(PmChart);

        QMetaObject::connectSlotsByName(PmChart);
    } // setupUi

    void retranslateUi(QMainWindow *PmChart)
    {
        PmChart->setWindowTitle(QCoreApplication::translate("PmChart", "PCP Charts", nullptr));
        fileOpenViewAction->setText(QCoreApplication::translate("PmChart", "&Open View...", nullptr));
        fileOpenViewAction->setIconText(QCoreApplication::translate("PmChart", "Open View", nullptr));
#if QT_CONFIG(shortcut)
        fileOpenViewAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        fileSaveViewAction->setText(QCoreApplication::translate("PmChart", "&Save View ...", nullptr));
        fileSaveViewAction->setIconText(QCoreApplication::translate("PmChart", "Save View", nullptr));
#if QT_CONFIG(shortcut)
        fileSaveViewAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        fileExportAction->setText(QCoreApplication::translate("PmChart", "&Export...", nullptr));
        fileExportAction->setIconText(QCoreApplication::translate("PmChart", "Export", nullptr));
#if QT_CONFIG(shortcut)
        fileExportAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+E", nullptr));
#endif // QT_CONFIG(shortcut)
        filePrintAction->setText(QCoreApplication::translate("PmChart", "&Print...", nullptr));
        filePrintAction->setIconText(QCoreApplication::translate("PmChart", "Print", nullptr));
#if QT_CONFIG(shortcut)
        filePrintAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
        fileQuitAction->setText(QCoreApplication::translate("PmChart", "&Quit", nullptr));
        fileQuitAction->setIconText(QCoreApplication::translate("PmChart", "Quit", nullptr));
#if QT_CONFIG(shortcut)
        fileQuitAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        newPmchartAction->setText(QCoreApplication::translate("PmChart", "New &pmchart", nullptr));
        newPmchartAction->setIconText(QCoreApplication::translate("PmChart", "New pmchart", nullptr));
        showTimeControlAction->setText(QCoreApplication::translate("PmChart", "Show Time", nullptr));
        hideTimeControlAction->setText(QCoreApplication::translate("PmChart", "Hide Time", nullptr));
        toolbarAction->setText(QCoreApplication::translate("PmChart", "Toolbar", nullptr));
        consoleAction->setText(QCoreApplication::translate("PmChart", "Console", nullptr));
        editChartAction->setText(QCoreApplication::translate("PmChart", "&Chart...", nullptr));
        editChartAction->setIconText(QCoreApplication::translate("PmChart", "Edit Chart", nullptr));
        editTabAction->setText(QCoreApplication::translate("PmChart", "Tab...", nullptr));
        editTabAction->setIconText(QCoreApplication::translate("PmChart", "Edit Tab", nullptr));
        editSamplesAction->setText(QCoreApplication::translate("PmChart", "Samples...", nullptr));
        editSamplesAction->setIconText(QCoreApplication::translate("PmChart", "Edit Samples", nullptr));
        editSettingsAction->setText(QCoreApplication::translate("PmChart", "Preferences...", nullptr));
        editSettingsAction->setIconText(QCoreApplication::translate("PmChart", "Preferences", nullptr));
        closeTabAction->setText(QCoreApplication::translate("PmChart", "Close Tab", nullptr));
        closeTabAction->setIconText(QCoreApplication::translate("PmChart", "Close Tab", nullptr));
#if QT_CONFIG(shortcut)
        closeTabAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+W", nullptr));
#endif // QT_CONFIG(shortcut)
        fileNewChartAction->setText(QCoreApplication::translate("PmChart", "&New Chart...", nullptr));
        fileNewChartAction->setIconText(QCoreApplication::translate("PmChart", "New Chart", nullptr));
        addTabAction->setText(QCoreApplication::translate("PmChart", "Add Tab...", nullptr));
        addTabAction->setIconText(QCoreApplication::translate("PmChart", "Add Tab", nullptr));
#if QT_CONFIG(shortcut)
        addTabAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+T", nullptr));
#endif // QT_CONFIG(shortcut)
        closeChartAction->setText(QCoreApplication::translate("PmChart", "Close Chart", nullptr));
        closeChartAction->setIconText(QCoreApplication::translate("PmChart", "Close Chart", nullptr));
#if QT_CONFIG(shortcut)
        closeChartAction->setShortcut(QCoreApplication::translate("PmChart", "Ctrl+D", nullptr));
#endif // QT_CONFIG(shortcut)
        zoomInAction->setText(QCoreApplication::translate("PmChart", "Zoom &In", nullptr));
        zoomInAction->setIconText(QCoreApplication::translate("PmChart", "Zoom In", nullptr));
        zoomOutAction->setText(QCoreApplication::translate("PmChart", "&Zoom Out", nullptr));
        zoomOutAction->setIconText(QCoreApplication::translate("PmChart", "Zoom Out", nullptr));
        helpManualAction->setText(QCoreApplication::translate("PmChart", "&Manual", nullptr));
        helpManualAction->setIconText(QCoreApplication::translate("PmChart", "Manual", nullptr));
#if QT_CONFIG(shortcut)
        helpManualAction->setShortcut(QCoreApplication::translate("PmChart", "F1", nullptr));
#endif // QT_CONFIG(shortcut)
        helpAboutAction->setText(QCoreApplication::translate("PmChart", "About", nullptr));
        helpAboutAction->setIconText(QCoreApplication::translate("PmChart", "About", nullptr));
#if QT_CONFIG(shortcut)
        helpAboutAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        helpAboutQtAction->setText(QCoreApplication::translate("PmChart", "About Qt", nullptr));
        helpAboutQtAction->setIconText(QCoreApplication::translate("PmChart", "About Qt", nullptr));
        helpSeeAlsoAction->setText(QCoreApplication::translate("PmChart", "See Also", nullptr));
        helpSeeAlsoAction->setIconText(QCoreApplication::translate("PmChart", "See Also", nullptr));
        helpWhatsThisAction->setText(QCoreApplication::translate("PmChart", "What's This", nullptr));
        helpWhatsThisAction->setIconText(QCoreApplication::translate("PmChart", "What's This", nullptr));
#if QT_CONFIG(shortcut)
        helpWhatsThisAction->setShortcut(QCoreApplication::translate("PmChart", "Shift+F1", nullptr));
#endif // QT_CONFIG(shortcut)
        recordStartAction->setText(QCoreApplication::translate("PmChart", "Start...", nullptr));
        recordQueryAction->setText(QCoreApplication::translate("PmChart", "Query", nullptr));
        recordStopAction->setText(QCoreApplication::translate("PmChart", "Stop", nullptr));
        recordDetachAction->setText(QCoreApplication::translate("PmChart", "Detach", nullptr));
#if QT_CONFIG(whatsthis)
        chartTabWidget->setWhatsThis(QCoreApplication::translate("PmChart", "The main chart canvas, displaying all charts in the current Tab.", nullptr));
#endif // QT_CONFIG(whatsthis)
        Help->setTitle(QCoreApplication::translate("PmChart", "&Help", nullptr));
        Time->setTitle(QCoreApplication::translate("PmChart", "&Options", nullptr));
        Record->setTitle(QCoreApplication::translate("PmChart", "&Record", nullptr));
        File->setTitle(QCoreApplication::translate("PmChart", "&File", nullptr));
        Edit->setTitle(QCoreApplication::translate("PmChart", "&Edit", nullptr));
#if QT_CONFIG(whatsthis)
        toolBar->setWhatsThis(QCoreApplication::translate("PmChart", "Configurable toolbar, use the Preferences dialog to change its contents", nullptr));
#endif // QT_CONFIG(whatsthis)
    } // retranslateUi

};

namespace Ui {
    class PmChart: public Ui_PmChart {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PMCHART_H
