/****************************************************************************
** Meta object code from reading C++ file 'pmtimelive.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "pmtimelive.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pmtimelive.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PmTimeLive_t {
    QByteArrayData data[26];
    char stringdata0[282];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PmTimeLive_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PmTimeLive_t qt_meta_stringdata_PmTimeLive = {
    {
QT_MOC_LITERAL(0, 0, 10), // "PmTimeLive"
QT_MOC_LITERAL(1, 11, 9), // "timePulse"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 15), // "PmTime::Packet*"
QT_MOC_LITERAL(4, 38, 12), // "vcrModePulse"
QT_MOC_LITERAL(5, 51, 7), // "tzPulse"
QT_MOC_LITERAL(6, 59, 5), // "char*"
QT_MOC_LITERAL(7, 65, 10), // "setControl"
QT_MOC_LITERAL(8, 76, 13), // "PmTime::State"
QT_MOC_LITERAL(9, 90, 8), // "newstate"
QT_MOC_LITERAL(10, 99, 4), // "init"
QT_MOC_LITERAL(11, 104, 4), // "quit"
QT_MOC_LITERAL(12, 109, 12), // "play_clicked"
QT_MOC_LITERAL(13, 122, 12), // "stop_clicked"
QT_MOC_LITERAL(14, 135, 10), // "updateTime"
QT_MOC_LITERAL(15, 146, 13), // "clickShowMsec"
QT_MOC_LITERAL(16, 160, 13), // "clickShowYear"
QT_MOC_LITERAL(17, 174, 11), // "changeDelta"
QT_MOC_LITERAL(18, 186, 5), // "value"
QT_MOC_LITERAL(19, 192, 14), // "disableConsole"
QT_MOC_LITERAL(20, 207, 21), // "lineEditDelta_changed"
QT_MOC_LITERAL(21, 229, 1), // "s"
QT_MOC_LITERAL(22, 231, 22), // "lineEditDelta_validate"
QT_MOC_LITERAL(23, 254, 11), // "setTimezone"
QT_MOC_LITERAL(24, 266, 8), // "QAction*"
QT_MOC_LITERAL(25, 275, 6) // "action"

    },
    "PmTimeLive\0timePulse\0\0PmTime::Packet*\0"
    "vcrModePulse\0tzPulse\0char*\0setControl\0"
    "PmTime::State\0newstate\0init\0quit\0"
    "play_clicked\0stop_clicked\0updateTime\0"
    "clickShowMsec\0clickShowYear\0changeDelta\0"
    "value\0disableConsole\0lineEditDelta_changed\0"
    "s\0lineEditDelta_validate\0setTimezone\0"
    "QAction*\0action"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PmTimeLive[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x06 /* Public */,
       4,    2,   97,    2, 0x06 /* Public */,
       5,    5,  102,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  113,    2, 0x0a /* Public */,
      10,    0,  116,    2, 0x0a /* Public */,
      11,    0,  117,    2, 0x0a /* Public */,
      12,    0,  118,    2, 0x0a /* Public */,
      13,    0,  119,    2, 0x0a /* Public */,
      14,    0,  120,    2, 0x0a /* Public */,
      15,    0,  121,    2, 0x0a /* Public */,
      16,    0,  122,    2, 0x0a /* Public */,
      17,    1,  123,    2, 0x0a /* Public */,
      19,    0,  126,    2, 0x0a /* Public */,
      20,    1,  127,    2, 0x0a /* Public */,
      22,    0,  130,    2, 0x0a /* Public */,
      23,    1,  131,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,    2,    2,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 6, QMetaType::Int, 0x80000000 | 6, QMetaType::Int,    2,    2,    2,    2,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   21,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 24,   25,

       0        // eod
};

void PmTimeLive::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PmTimeLive *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->timePulse((*reinterpret_cast< PmTime::Packet*(*)>(_a[1]))); break;
        case 1: _t->vcrModePulse((*reinterpret_cast< PmTime::Packet*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->tzPulse((*reinterpret_cast< PmTime::Packet*(*)>(_a[1])),(*reinterpret_cast< char*(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< char*(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 3: _t->setControl((*reinterpret_cast< PmTime::State(*)>(_a[1]))); break;
        case 4: _t->init(); break;
        case 5: _t->quit(); break;
        case 6: _t->play_clicked(); break;
        case 7: _t->stop_clicked(); break;
        case 8: _t->updateTime(); break;
        case 9: _t->clickShowMsec(); break;
        case 10: _t->clickShowYear(); break;
        case 11: _t->changeDelta((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->disableConsole(); break;
        case 13: _t->lineEditDelta_changed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->lineEditDelta_validate(); break;
        case 15: _t->setTimezone((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PmTimeLive::*)(PmTime::Packet * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PmTimeLive::timePulse)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PmTimeLive::*)(PmTime::Packet * , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PmTimeLive::vcrModePulse)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PmTimeLive::*)(PmTime::Packet * , char * , int , char * , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PmTimeLive::tzPulse)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PmTimeLive::staticMetaObject = { {
    QMetaObject::SuperData::link<PmTime::staticMetaObject>(),
    qt_meta_stringdata_PmTimeLive.data,
    qt_meta_data_PmTimeLive,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PmTimeLive::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PmTimeLive::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PmTimeLive.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PmTimeLive"))
        return static_cast< Ui::PmTimeLive*>(this);
    return PmTime::qt_metacast(_clname);
}

int PmTimeLive::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PmTime::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void PmTimeLive::timePulse(PmTime::Packet * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PmTimeLive::vcrModePulse(PmTime::Packet * _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PmTimeLive::tzPulse(PmTime::Packet * _t1, char * _t2, int _t3, char * _t4, int _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
