/********************************************************************************
** Form generated from reading UI file 'settingsdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSDIALOG_H
#define UI_SETTINGSDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "colorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_SettingsDialog
{
public:
    QTabWidget *settingsTab;
    QWidget *samplesTab;
    QWidget *samplesLayoutWidget;
    QGridLayout *gridLayout;
    QLabel *loggerDeltaLabel;
    QLabel *chartDeltaLabel;
    QSlider *visibleSlider;
    QLabel *visibleTextLabel;
    QLabel *sampleTextLabel;
    QSpinBox *sampleCounter;
    QLineEdit *chartDeltaLineEdit;
    QSpinBox *visibleCounter;
    QSlider *sampleSlider;
    QComboBox *chartDeltaUnitsComboBox;
    QComboBox *loggerDeltaUnitsComboBox;
    QLineEdit *loggerDeltaLineEdit;
    QWidget *colorTab;
    QGroupBox *colorSchemesGroupBox;
    ColorButton *colorButton1;
    ColorButton *colorButton2;
    ColorButton *colorButton3;
    ColorButton *colorButton4;
    ColorButton *colorButton5;
    ColorButton *colorButton6;
    ColorButton *colorButton7;
    ColorButton *colorButton8;
    ColorButton *colorButton9;
    ColorButton *colorButton10;
    ColorButton *colorButton11;
    ColorButton *colorButton12;
    ColorButton *colorButton13;
    ColorButton *colorButton14;
    ColorButton *colorButton15;
    ColorButton *colorButton16;
    ColorButton *colorButton17;
    ColorButton *colorButton18;
    ColorButton *colorButton19;
    ColorButton *colorButton20;
    ColorButton *colorButton21;
    ColorButton *colorButton22;
    QComboBox *schemeComboBox;
    QLineEdit *schemeLineEdit;
    QPushButton *updateSchemeButton;
    QPushButton *removeSchemeButton;
    ColorButton *selectedHighlightButton;
    ColorButton *defaultBackgroundButton;
    QLabel *selectedHighlightLabel;
    QLabel *defaultBackgroundLabel;
    QWidget *fontTab;
    QWidget *sizeLayoutWidget;
    QVBoxLayout *sizeVerticalLayout;
    QLabel *sizeLabel;
    QLineEdit *sizeLineEdit;
    QListWidget *sizeListWidget;
    QWidget *styleLayoutWidget;
    QVBoxLayout *styleVerticalLayout;
    QLabel *styleLabel;
    QLineEdit *styleLineEdit;
    QListWidget *styleListWidget;
    QWidget *familyLayoutWidget;
    QVBoxLayout *familyVerticalLayout;
    QLabel *familyLabel;
    QLineEdit *familyLineEdit;
    QListWidget *familyListWidget;
    QPushButton *resetFontButton;
    QPushButton *applyFontButton;
    QWidget *tab;
    QPushButton *addHostButton;
    QPushButton *removeHostButton;
    QPushButton *hostButton;
    QComboBox *hostComboBox;
    QGroupBox *savedHostsGroupBox;
    QListWidget *savedHostsListWidget;
    QWidget *toolbarTab;
    QGridLayout *gridLayout1;
    QVBoxLayout *vboxLayout;
    QCheckBox *startupToolbarCheckBox;
    QCheckBox *nativeToolbarCheckBox;
    QHBoxLayout *hboxLayout;
    QLabel *toolbarLocationLabel;
    QComboBox *toolbarAreasComboBox;
    QLabel *toolbarActionsLabel;
    QListWidget *actionListWidget;
    QPushButton *closeSettingsButton;

    void setupUi(QDialog *SettingsDialog)
    {
        if (SettingsDialog->objectName().isEmpty())
            SettingsDialog->setObjectName(QString::fromUtf8("SettingsDialog"));
        SettingsDialog->resize(390, 300);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SettingsDialog->sizePolicy().hasHeightForWidth());
        SettingsDialog->setSizePolicy(sizePolicy);
        SettingsDialog->setMinimumSize(QSize(390, 300));
        SettingsDialog->setMaximumSize(QSize(390, 300));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/settings.png"), QSize(), QIcon::Normal, QIcon::Off);
        SettingsDialog->setWindowIcon(icon);
        SettingsDialog->setSizeGripEnabled(false);
        settingsTab = new QTabWidget(SettingsDialog);
        settingsTab->setObjectName(QString::fromUtf8("settingsTab"));
        settingsTab->setGeometry(QRect(10, 10, 371, 241));
        samplesTab = new QWidget();
        samplesTab->setObjectName(QString::fromUtf8("samplesTab"));
        samplesLayoutWidget = new QWidget(samplesTab);
        samplesLayoutWidget->setObjectName(QString::fromUtf8("samplesLayoutWidget"));
        samplesLayoutWidget->setGeometry(QRect(10, 5, 351, 171));
        gridLayout = new QGridLayout(samplesLayoutWidget);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        loggerDeltaLabel = new QLabel(samplesLayoutWidget);
        loggerDeltaLabel->setObjectName(QString::fromUtf8("loggerDeltaLabel"));

        gridLayout->addWidget(loggerDeltaLabel, 1, 0, 1, 2);

        chartDeltaLabel = new QLabel(samplesLayoutWidget);
        chartDeltaLabel->setObjectName(QString::fromUtf8("chartDeltaLabel"));

        gridLayout->addWidget(chartDeltaLabel, 0, 0, 1, 2);

        visibleSlider = new QSlider(samplesLayoutWidget);
        visibleSlider->setObjectName(QString::fromUtf8("visibleSlider"));
        visibleSlider->setMaximum(999999999);
        visibleSlider->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(visibleSlider, 2, 1, 1, 3);

        visibleTextLabel = new QLabel(samplesLayoutWidget);
        visibleTextLabel->setObjectName(QString::fromUtf8("visibleTextLabel"));
        visibleTextLabel->setWordWrap(false);

        gridLayout->addWidget(visibleTextLabel, 2, 0, 1, 1);

        sampleTextLabel = new QLabel(samplesLayoutWidget);
        sampleTextLabel->setObjectName(QString::fromUtf8("sampleTextLabel"));
        sampleTextLabel->setWordWrap(false);

        gridLayout->addWidget(sampleTextLabel, 3, 0, 1, 1);

        sampleCounter = new QSpinBox(samplesLayoutWidget);
        sampleCounter->setObjectName(QString::fromUtf8("sampleCounter"));
        sampleCounter->setMaximum(999999999);

        gridLayout->addWidget(sampleCounter, 3, 4, 1, 1);

        chartDeltaLineEdit = new QLineEdit(samplesLayoutWidget);
        chartDeltaLineEdit->setObjectName(QString::fromUtf8("chartDeltaLineEdit"));
        chartDeltaLineEdit->setMaxLength(6);

        gridLayout->addWidget(chartDeltaLineEdit, 0, 2, 1, 1);

        visibleCounter = new QSpinBox(samplesLayoutWidget);
        visibleCounter->setObjectName(QString::fromUtf8("visibleCounter"));
        visibleCounter->setMaximum(999999999);

        gridLayout->addWidget(visibleCounter, 2, 4, 1, 1);

        sampleSlider = new QSlider(samplesLayoutWidget);
        sampleSlider->setObjectName(QString::fromUtf8("sampleSlider"));
        sampleSlider->setMaximum(999999999);
        sampleSlider->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(sampleSlider, 3, 1, 1, 3);

        chartDeltaUnitsComboBox = new QComboBox(samplesLayoutWidget);
        chartDeltaUnitsComboBox->addItem(QString());
        chartDeltaUnitsComboBox->addItem(QString());
        chartDeltaUnitsComboBox->addItem(QString());
        chartDeltaUnitsComboBox->addItem(QString());
        chartDeltaUnitsComboBox->addItem(QString());
        chartDeltaUnitsComboBox->addItem(QString());
        chartDeltaUnitsComboBox->setObjectName(QString::fromUtf8("chartDeltaUnitsComboBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(chartDeltaUnitsComboBox->sizePolicy().hasHeightForWidth());
        chartDeltaUnitsComboBox->setSizePolicy(sizePolicy1);
        chartDeltaUnitsComboBox->setMaxCount(6);
        chartDeltaUnitsComboBox->setInsertPolicy(QComboBox::NoInsert);
        chartDeltaUnitsComboBox->setDuplicatesEnabled(false);

        gridLayout->addWidget(chartDeltaUnitsComboBox, 0, 3, 1, 2);

        loggerDeltaUnitsComboBox = new QComboBox(samplesLayoutWidget);
        loggerDeltaUnitsComboBox->addItem(QString());
        loggerDeltaUnitsComboBox->addItem(QString());
        loggerDeltaUnitsComboBox->addItem(QString());
        loggerDeltaUnitsComboBox->addItem(QString());
        loggerDeltaUnitsComboBox->addItem(QString());
        loggerDeltaUnitsComboBox->addItem(QString());
        loggerDeltaUnitsComboBox->setObjectName(QString::fromUtf8("loggerDeltaUnitsComboBox"));
        sizePolicy1.setHeightForWidth(loggerDeltaUnitsComboBox->sizePolicy().hasHeightForWidth());
        loggerDeltaUnitsComboBox->setSizePolicy(sizePolicy1);
        loggerDeltaUnitsComboBox->setMaxCount(6);
        loggerDeltaUnitsComboBox->setInsertPolicy(QComboBox::NoInsert);
        loggerDeltaUnitsComboBox->setDuplicatesEnabled(false);

        gridLayout->addWidget(loggerDeltaUnitsComboBox, 1, 3, 1, 2);

        loggerDeltaLineEdit = new QLineEdit(samplesLayoutWidget);
        loggerDeltaLineEdit->setObjectName(QString::fromUtf8("loggerDeltaLineEdit"));
        loggerDeltaLineEdit->setMaxLength(6);

        gridLayout->addWidget(loggerDeltaLineEdit, 1, 2, 1, 1);

        settingsTab->addTab(samplesTab, QString());
        colorTab = new QWidget();
        colorTab->setObjectName(QString::fromUtf8("colorTab"));
        colorSchemesGroupBox = new QGroupBox(colorTab);
        colorSchemesGroupBox->setObjectName(QString::fromUtf8("colorSchemesGroupBox"));
        colorSchemesGroupBox->setGeometry(QRect(10, 80, 351, 121));
        colorButton1 = new ColorButton(colorSchemesGroupBox);
        colorButton1->setObjectName(QString::fromUtf8("colorButton1"));
        colorButton1->setGeometry(QRect(10, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton1->sizePolicy().hasHeightForWidth());
        colorButton1->setSizePolicy(sizePolicy);
        colorButton1->setMinimumSize(QSize(30, 20));
        colorButton1->setMaximumSize(QSize(30, 20));
        colorButton2 = new ColorButton(colorSchemesGroupBox);
        colorButton2->setObjectName(QString::fromUtf8("colorButton2"));
        colorButton2->setGeometry(QRect(40, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton2->sizePolicy().hasHeightForWidth());
        colorButton2->setSizePolicy(sizePolicy);
        colorButton2->setMaximumSize(QSize(30, 20));
        colorButton3 = new ColorButton(colorSchemesGroupBox);
        colorButton3->setObjectName(QString::fromUtf8("colorButton3"));
        colorButton3->setGeometry(QRect(70, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton3->sizePolicy().hasHeightForWidth());
        colorButton3->setSizePolicy(sizePolicy);
        colorButton3->setMaximumSize(QSize(30, 20));
        colorButton4 = new ColorButton(colorSchemesGroupBox);
        colorButton4->setObjectName(QString::fromUtf8("colorButton4"));
        colorButton4->setGeometry(QRect(100, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton4->sizePolicy().hasHeightForWidth());
        colorButton4->setSizePolicy(sizePolicy);
        colorButton4->setMaximumSize(QSize(30, 20));
        colorButton5 = new ColorButton(colorSchemesGroupBox);
        colorButton5->setObjectName(QString::fromUtf8("colorButton5"));
        colorButton5->setGeometry(QRect(130, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton5->sizePolicy().hasHeightForWidth());
        colorButton5->setSizePolicy(sizePolicy);
        colorButton5->setMinimumSize(QSize(30, 20));
        colorButton5->setMaximumSize(QSize(30, 20));
        colorButton6 = new ColorButton(colorSchemesGroupBox);
        colorButton6->setObjectName(QString::fromUtf8("colorButton6"));
        colorButton6->setGeometry(QRect(160, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton6->sizePolicy().hasHeightForWidth());
        colorButton6->setSizePolicy(sizePolicy);
        colorButton6->setMinimumSize(QSize(30, 20));
        colorButton6->setMaximumSize(QSize(30, 20));
        colorButton7 = new ColorButton(colorSchemesGroupBox);
        colorButton7->setObjectName(QString::fromUtf8("colorButton7"));
        colorButton7->setGeometry(QRect(190, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton7->sizePolicy().hasHeightForWidth());
        colorButton7->setSizePolicy(sizePolicy);
        colorButton7->setMinimumSize(QSize(30, 20));
        colorButton7->setMaximumSize(QSize(30, 20));
        colorButton8 = new ColorButton(colorSchemesGroupBox);
        colorButton8->setObjectName(QString::fromUtf8("colorButton8"));
        colorButton8->setGeometry(QRect(220, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton8->sizePolicy().hasHeightForWidth());
        colorButton8->setSizePolicy(sizePolicy);
        colorButton8->setMinimumSize(QSize(30, 20));
        colorButton8->setMaximumSize(QSize(30, 20));
        colorButton9 = new ColorButton(colorSchemesGroupBox);
        colorButton9->setObjectName(QString::fromUtf8("colorButton9"));
        colorButton9->setGeometry(QRect(250, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton9->sizePolicy().hasHeightForWidth());
        colorButton9->setSizePolicy(sizePolicy);
        colorButton9->setMinimumSize(QSize(30, 20));
        colorButton9->setMaximumSize(QSize(30, 20));
        colorButton10 = new ColorButton(colorSchemesGroupBox);
        colorButton10->setObjectName(QString::fromUtf8("colorButton10"));
        colorButton10->setGeometry(QRect(280, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton10->sizePolicy().hasHeightForWidth());
        colorButton10->setSizePolicy(sizePolicy);
        colorButton10->setMinimumSize(QSize(30, 20));
        colorButton10->setMaximumSize(QSize(30, 20));
        colorButton11 = new ColorButton(colorSchemesGroupBox);
        colorButton11->setObjectName(QString::fromUtf8("colorButton11"));
        colorButton11->setGeometry(QRect(310, 50, 30, 20));
        sizePolicy.setHeightForWidth(colorButton11->sizePolicy().hasHeightForWidth());
        colorButton11->setSizePolicy(sizePolicy);
        colorButton11->setMinimumSize(QSize(30, 20));
        colorButton11->setMaximumSize(QSize(30, 20));
        colorButton12 = new ColorButton(colorSchemesGroupBox);
        colorButton12->setObjectName(QString::fromUtf8("colorButton12"));
        colorButton12->setGeometry(QRect(10, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton12->sizePolicy().hasHeightForWidth());
        colorButton12->setSizePolicy(sizePolicy);
        colorButton12->setMinimumSize(QSize(30, 20));
        colorButton12->setMaximumSize(QSize(30, 20));
        colorButton13 = new ColorButton(colorSchemesGroupBox);
        colorButton13->setObjectName(QString::fromUtf8("colorButton13"));
        colorButton13->setGeometry(QRect(40, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton13->sizePolicy().hasHeightForWidth());
        colorButton13->setSizePolicy(sizePolicy);
        colorButton13->setMinimumSize(QSize(30, 20));
        colorButton13->setMaximumSize(QSize(30, 20));
        colorButton14 = new ColorButton(colorSchemesGroupBox);
        colorButton14->setObjectName(QString::fromUtf8("colorButton14"));
        colorButton14->setGeometry(QRect(70, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton14->sizePolicy().hasHeightForWidth());
        colorButton14->setSizePolicy(sizePolicy);
        colorButton14->setMinimumSize(QSize(30, 20));
        colorButton14->setMaximumSize(QSize(30, 20));
        colorButton15 = new ColorButton(colorSchemesGroupBox);
        colorButton15->setObjectName(QString::fromUtf8("colorButton15"));
        colorButton15->setGeometry(QRect(100, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton15->sizePolicy().hasHeightForWidth());
        colorButton15->setSizePolicy(sizePolicy);
        colorButton15->setMinimumSize(QSize(30, 20));
        colorButton15->setMaximumSize(QSize(30, 20));
        colorButton16 = new ColorButton(colorSchemesGroupBox);
        colorButton16->setObjectName(QString::fromUtf8("colorButton16"));
        colorButton16->setGeometry(QRect(130, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton16->sizePolicy().hasHeightForWidth());
        colorButton16->setSizePolicy(sizePolicy);
        colorButton16->setMinimumSize(QSize(30, 20));
        colorButton16->setMaximumSize(QSize(30, 20));
        colorButton17 = new ColorButton(colorSchemesGroupBox);
        colorButton17->setObjectName(QString::fromUtf8("colorButton17"));
        colorButton17->setGeometry(QRect(160, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton17->sizePolicy().hasHeightForWidth());
        colorButton17->setSizePolicy(sizePolicy);
        colorButton17->setMinimumSize(QSize(30, 20));
        colorButton17->setMaximumSize(QSize(30, 20));
        colorButton18 = new ColorButton(colorSchemesGroupBox);
        colorButton18->setObjectName(QString::fromUtf8("colorButton18"));
        colorButton18->setGeometry(QRect(190, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton18->sizePolicy().hasHeightForWidth());
        colorButton18->setSizePolicy(sizePolicy);
        colorButton18->setMinimumSize(QSize(30, 20));
        colorButton18->setMaximumSize(QSize(30, 20));
        colorButton19 = new ColorButton(colorSchemesGroupBox);
        colorButton19->setObjectName(QString::fromUtf8("colorButton19"));
        colorButton19->setGeometry(QRect(220, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton19->sizePolicy().hasHeightForWidth());
        colorButton19->setSizePolicy(sizePolicy);
        colorButton19->setMinimumSize(QSize(30, 20));
        colorButton19->setMaximumSize(QSize(30, 20));
        colorButton20 = new ColorButton(colorSchemesGroupBox);
        colorButton20->setObjectName(QString::fromUtf8("colorButton20"));
        colorButton20->setGeometry(QRect(250, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton20->sizePolicy().hasHeightForWidth());
        colorButton20->setSizePolicy(sizePolicy);
        colorButton20->setMinimumSize(QSize(30, 20));
        colorButton20->setMaximumSize(QSize(30, 20));
        colorButton21 = new ColorButton(colorSchemesGroupBox);
        colorButton21->setObjectName(QString::fromUtf8("colorButton21"));
        colorButton21->setGeometry(QRect(280, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton21->sizePolicy().hasHeightForWidth());
        colorButton21->setSizePolicy(sizePolicy);
        colorButton21->setMinimumSize(QSize(30, 20));
        colorButton21->setMaximumSize(QSize(30, 20));
        colorButton22 = new ColorButton(colorSchemesGroupBox);
        colorButton22->setObjectName(QString::fromUtf8("colorButton22"));
        colorButton22->setGeometry(QRect(310, 70, 30, 20));
        sizePolicy.setHeightForWidth(colorButton22->sizePolicy().hasHeightForWidth());
        colorButton22->setSizePolicy(sizePolicy);
        colorButton22->setMinimumSize(QSize(30, 20));
        colorButton22->setMaximumSize(QSize(30, 20));
        schemeComboBox = new QComboBox(colorSchemesGroupBox);
        schemeComboBox->setObjectName(QString::fromUtf8("schemeComboBox"));
        schemeComboBox->setGeometry(QRect(11, 22, 171, 26));
        schemeLineEdit = new QLineEdit(colorSchemesGroupBox);
        schemeLineEdit->setObjectName(QString::fromUtf8("schemeLineEdit"));
        schemeLineEdit->setEnabled(false);
        schemeLineEdit->setGeometry(QRect(190, 20, 151, 29));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(schemeLineEdit->sizePolicy().hasHeightForWidth());
        schemeLineEdit->setSizePolicy(sizePolicy2);
        updateSchemeButton = new QPushButton(colorSchemesGroupBox);
        updateSchemeButton->setObjectName(QString::fromUtf8("updateSchemeButton"));
        updateSchemeButton->setGeometry(QRect(200, 100, 75, 21));
        removeSchemeButton = new QPushButton(colorSchemesGroupBox);
        removeSchemeButton->setObjectName(QString::fromUtf8("removeSchemeButton"));
        removeSchemeButton->setGeometry(QRect(70, 100, 75, 21));
        selectedHighlightButton = new ColorButton(colorTab);
        selectedHighlightButton->setObjectName(QString::fromUtf8("selectedHighlightButton"));
        selectedHighlightButton->setGeometry(QRect(200, 40, 30, 20));
        sizePolicy.setHeightForWidth(selectedHighlightButton->sizePolicy().hasHeightForWidth());
        selectedHighlightButton->setSizePolicy(sizePolicy);
        selectedHighlightButton->setMinimumSize(QSize(30, 20));
        selectedHighlightButton->setMaximumSize(QSize(30, 20));
        defaultBackgroundButton = new ColorButton(colorTab);
        defaultBackgroundButton->setObjectName(QString::fromUtf8("defaultBackgroundButton"));
        defaultBackgroundButton->setGeometry(QRect(200, 10, 30, 20));
        sizePolicy.setHeightForWidth(defaultBackgroundButton->sizePolicy().hasHeightForWidth());
        defaultBackgroundButton->setSizePolicy(sizePolicy);
        defaultBackgroundButton->setMinimumSize(QSize(30, 20));
        defaultBackgroundButton->setMaximumSize(QSize(30, 20));
        selectedHighlightLabel = new QLabel(colorTab);
        selectedHighlightLabel->setObjectName(QString::fromUtf8("selectedHighlightLabel"));
        selectedHighlightLabel->setGeometry(QRect(10, 30, 171, 41));
        selectedHighlightLabel->setWordWrap(false);
        defaultBackgroundLabel = new QLabel(colorTab);
        defaultBackgroundLabel->setObjectName(QString::fromUtf8("defaultBackgroundLabel"));
        defaultBackgroundLabel->setGeometry(QRect(10, 0, 171, 41));
        defaultBackgroundLabel->setWordWrap(false);
        settingsTab->addTab(colorTab, QString());
        fontTab = new QWidget();
        fontTab->setObjectName(QString::fromUtf8("fontTab"));
        sizeLayoutWidget = new QWidget(fontTab);
        sizeLayoutWidget->setObjectName(QString::fromUtf8("sizeLayoutWidget"));
        sizeLayoutWidget->setGeometry(QRect(310, 10, 48, 161));
        sizeVerticalLayout = new QVBoxLayout(sizeLayoutWidget);
        sizeVerticalLayout->setSpacing(3);
        sizeVerticalLayout->setObjectName(QString::fromUtf8("sizeVerticalLayout"));
        sizeVerticalLayout->setContentsMargins(0, 0, 0, 0);
        sizeLabel = new QLabel(sizeLayoutWidget);
        sizeLabel->setObjectName(QString::fromUtf8("sizeLabel"));

        sizeVerticalLayout->addWidget(sizeLabel);

        sizeLineEdit = new QLineEdit(sizeLayoutWidget);
        sizeLineEdit->setObjectName(QString::fromUtf8("sizeLineEdit"));

        sizeVerticalLayout->addWidget(sizeLineEdit);

        sizeListWidget = new QListWidget(sizeLayoutWidget);
        sizeListWidget->setObjectName(QString::fromUtf8("sizeListWidget"));

        sizeVerticalLayout->addWidget(sizeListWidget);

        styleLayoutWidget = new QWidget(fontTab);
        styleLayoutWidget->setObjectName(QString::fromUtf8("styleLayoutWidget"));
        styleLayoutWidget->setGeometry(QRect(220, 10, 82, 161));
        styleVerticalLayout = new QVBoxLayout(styleLayoutWidget);
        styleVerticalLayout->setSpacing(3);
        styleVerticalLayout->setObjectName(QString::fromUtf8("styleVerticalLayout"));
        styleVerticalLayout->setContentsMargins(0, 0, 0, 0);
        styleLabel = new QLabel(styleLayoutWidget);
        styleLabel->setObjectName(QString::fromUtf8("styleLabel"));

        styleVerticalLayout->addWidget(styleLabel);

        styleLineEdit = new QLineEdit(styleLayoutWidget);
        styleLineEdit->setObjectName(QString::fromUtf8("styleLineEdit"));

        styleVerticalLayout->addWidget(styleLineEdit);

        styleListWidget = new QListWidget(styleLayoutWidget);
        styleListWidget->setObjectName(QString::fromUtf8("styleListWidget"));

        styleVerticalLayout->addWidget(styleListWidget);

        familyLayoutWidget = new QWidget(fontTab);
        familyLayoutWidget->setObjectName(QString::fromUtf8("familyLayoutWidget"));
        familyLayoutWidget->setGeometry(QRect(10, 10, 205, 161));
        familyVerticalLayout = new QVBoxLayout(familyLayoutWidget);
        familyVerticalLayout->setSpacing(3);
        familyVerticalLayout->setObjectName(QString::fromUtf8("familyVerticalLayout"));
        familyVerticalLayout->setContentsMargins(0, 0, 0, 0);
        familyLabel = new QLabel(familyLayoutWidget);
        familyLabel->setObjectName(QString::fromUtf8("familyLabel"));

        familyVerticalLayout->addWidget(familyLabel);

        familyLineEdit = new QLineEdit(familyLayoutWidget);
        familyLineEdit->setObjectName(QString::fromUtf8("familyLineEdit"));

        familyVerticalLayout->addWidget(familyLineEdit);

        familyListWidget = new QListWidget(familyLayoutWidget);
        familyListWidget->setObjectName(QString::fromUtf8("familyListWidget"));

        familyVerticalLayout->addWidget(familyListWidget);

        resetFontButton = new QPushButton(fontTab);
        resetFontButton->setObjectName(QString::fromUtf8("resetFontButton"));
        resetFontButton->setGeometry(QRect(80, 180, 75, 21));
        applyFontButton = new QPushButton(fontTab);
        applyFontButton->setObjectName(QString::fromUtf8("applyFontButton"));
        applyFontButton->setGeometry(QRect(210, 180, 75, 21));
        settingsTab->addTab(fontTab, QString());
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        addHostButton = new QPushButton(tab);
        addHostButton->setObjectName(QString::fromUtf8("addHostButton"));
        addHostButton->setGeometry(QRect(210, 180, 75, 21));
        removeHostButton = new QPushButton(tab);
        removeHostButton->setObjectName(QString::fromUtf8("removeHostButton"));
        removeHostButton->setGeometry(QRect(80, 180, 75, 21));
        hostButton = new QPushButton(tab);
        hostButton->setObjectName(QString::fromUtf8("hostButton"));
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/computer.png"));
        hostButton->setIcon(icon1);
        hostButton->setIconSize(QSize(16, 16));
        hostButton->setGeometry(QRect(260, 10, 92, 27));
        hostComboBox = new QComboBox(tab);
        hostComboBox->setObjectName(QString::fromUtf8("hostComboBox"));
        hostComboBox->setGeometry(QRect(10, 10, 241, 27));
        savedHostsGroupBox = new QGroupBox(tab);
        savedHostsGroupBox->setObjectName(QString::fromUtf8("savedHostsGroupBox"));
        savedHostsGroupBox->setGeometry(QRect(10, 40, 341, 131));
        savedHostsListWidget = new QListWidget(savedHostsGroupBox);
        savedHostsListWidget->setObjectName(QString::fromUtf8("savedHostsListWidget"));
        savedHostsListWidget->setGeometry(QRect(10, 20, 321, 111));
        savedHostsListWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
        savedHostsListWidget->setViewMode(QListView::ListMode);
        savedHostsListWidget->setSelectionRectVisible(false);
        savedHostsListWidget->setSortingEnabled(true);
        settingsTab->addTab(tab, QString());
        toolbarTab = new QWidget();
        toolbarTab->setObjectName(QString::fromUtf8("toolbarTab"));
        gridLayout1 = new QGridLayout(toolbarTab);
#ifndef Q_OS_MAC
        gridLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout1->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        startupToolbarCheckBox = new QCheckBox(toolbarTab);
        startupToolbarCheckBox->setObjectName(QString::fromUtf8("startupToolbarCheckBox"));
        startupToolbarCheckBox->setChecked(true);

        vboxLayout->addWidget(startupToolbarCheckBox);

        nativeToolbarCheckBox = new QCheckBox(toolbarTab);
        nativeToolbarCheckBox->setObjectName(QString::fromUtf8("nativeToolbarCheckBox"));
        nativeToolbarCheckBox->setChecked(true);

        vboxLayout->addWidget(nativeToolbarCheckBox);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        toolbarLocationLabel = new QLabel(toolbarTab);
        toolbarLocationLabel->setObjectName(QString::fromUtf8("toolbarLocationLabel"));

        hboxLayout->addWidget(toolbarLocationLabel);

        toolbarAreasComboBox = new QComboBox(toolbarTab);
        toolbarAreasComboBox->addItem(QString());
        toolbarAreasComboBox->addItem(QString());
        toolbarAreasComboBox->setObjectName(QString::fromUtf8("toolbarAreasComboBox"));

        hboxLayout->addWidget(toolbarAreasComboBox);


        vboxLayout->addLayout(hboxLayout);

        toolbarActionsLabel = new QLabel(toolbarTab);
        toolbarActionsLabel->setObjectName(QString::fromUtf8("toolbarActionsLabel"));

        vboxLayout->addWidget(toolbarActionsLabel);

        actionListWidget = new QListWidget(toolbarTab);
        actionListWidget->setObjectName(QString::fromUtf8("actionListWidget"));
        actionListWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        actionListWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        actionListWidget->setProperty("showDropIndicator", QVariant(false));
        actionListWidget->setDragDropMode(QAbstractItemView::DragDrop);
        actionListWidget->setIconSize(QSize(22, 22));
        actionListWidget->setTextElideMode(Qt::ElideNone);
        actionListWidget->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        actionListWidget->setFlow(QListView::LeftToRight);
        actionListWidget->setProperty("isWrapping", QVariant(false));
        actionListWidget->setSpacing(2);
        actionListWidget->setViewMode(QListView::IconMode);
        actionListWidget->setUniformItemSizes(true);

        vboxLayout->addWidget(actionListWidget);


        gridLayout1->addLayout(vboxLayout, 0, 0, 1, 1);

        settingsTab->addTab(toolbarTab, QString());
        closeSettingsButton = new QPushButton(SettingsDialog);
        closeSettingsButton->setObjectName(QString::fromUtf8("closeSettingsButton"));
        closeSettingsButton->setGeometry(QRect(290, 260, 92, 31));
        closeSettingsButton->setAutoDefault(true);

        retranslateUi(SettingsDialog);

        settingsTab->setCurrentIndex(1);
        chartDeltaUnitsComboBox->setCurrentIndex(1);
        loggerDeltaUnitsComboBox->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(SettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *SettingsDialog)
    {
        SettingsDialog->setWindowTitle(QCoreApplication::translate("SettingsDialog", "Settings", nullptr));
        loggerDeltaLabel->setText(QCoreApplication::translate("SettingsDialog", "Default Record Interval:", nullptr));
        chartDeltaLabel->setText(QCoreApplication::translate("SettingsDialog", "Default Chart Interval:", nullptr));
        visibleTextLabel->setText(QCoreApplication::translate("SettingsDialog", "Default Visible Points:", nullptr));
        sampleTextLabel->setText(QCoreApplication::translate("SettingsDialog", "Default Sample Count:", nullptr));
        chartDeltaUnitsComboBox->setItemText(0, QCoreApplication::translate("SettingsDialog", "Milliseconds", nullptr));
        chartDeltaUnitsComboBox->setItemText(1, QCoreApplication::translate("SettingsDialog", "Seconds", nullptr));
        chartDeltaUnitsComboBox->setItemText(2, QCoreApplication::translate("SettingsDialog", "Minutes", nullptr));
        chartDeltaUnitsComboBox->setItemText(3, QCoreApplication::translate("SettingsDialog", "Hours", nullptr));
        chartDeltaUnitsComboBox->setItemText(4, QCoreApplication::translate("SettingsDialog", "Days", nullptr));
        chartDeltaUnitsComboBox->setItemText(5, QCoreApplication::translate("SettingsDialog", "Weeks", nullptr));

        loggerDeltaUnitsComboBox->setItemText(0, QCoreApplication::translate("SettingsDialog", "Milliseconds", nullptr));
        loggerDeltaUnitsComboBox->setItemText(1, QCoreApplication::translate("SettingsDialog", "Seconds", nullptr));
        loggerDeltaUnitsComboBox->setItemText(2, QCoreApplication::translate("SettingsDialog", "Minutes", nullptr));
        loggerDeltaUnitsComboBox->setItemText(3, QCoreApplication::translate("SettingsDialog", "Hours", nullptr));
        loggerDeltaUnitsComboBox->setItemText(4, QCoreApplication::translate("SettingsDialog", "Days", nullptr));
        loggerDeltaUnitsComboBox->setItemText(5, QCoreApplication::translate("SettingsDialog", "Weeks", nullptr));

        settingsTab->setTabText(settingsTab->indexOf(samplesTab), QCoreApplication::translate("SettingsDialog", "Sampling", nullptr));
        colorSchemesGroupBox->setTitle(QCoreApplication::translate("SettingsDialog", "Color Schemes", nullptr));
        colorButton1->setText(QString());
        colorButton2->setText(QString());
        colorButton3->setText(QString());
        colorButton4->setText(QString());
        colorButton5->setText(QString());
        colorButton6->setText(QString());
        colorButton7->setText(QString());
        colorButton8->setText(QString());
        colorButton9->setText(QString());
        colorButton10->setText(QString());
        colorButton11->setText(QString());
        colorButton12->setText(QString());
        colorButton13->setText(QString());
        colorButton14->setText(QString());
        colorButton15->setText(QString());
        colorButton16->setText(QString());
        colorButton17->setText(QString());
        colorButton18->setText(QString());
        colorButton19->setText(QString());
        colorButton20->setText(QString());
        colorButton21->setText(QString());
        colorButton22->setText(QString());
        schemeLineEdit->setText(QCoreApplication::translate("SettingsDialog", "#-cycle", nullptr));
        updateSchemeButton->setText(QCoreApplication::translate("SettingsDialog", "Update", nullptr));
        removeSchemeButton->setText(QCoreApplication::translate("SettingsDialog", "Remove", nullptr));
        selectedHighlightButton->setText(QString());
        defaultBackgroundButton->setText(QString());
        selectedHighlightLabel->setText(QCoreApplication::translate("SettingsDialog", "Selected Chart Highlight:", nullptr));
        defaultBackgroundLabel->setText(QCoreApplication::translate("SettingsDialog", "Default Chart Background:", nullptr));
        settingsTab->setTabText(settingsTab->indexOf(colorTab), QCoreApplication::translate("SettingsDialog", "Colors", nullptr));
        sizeLabel->setText(QCoreApplication::translate("SettingsDialog", "Size", nullptr));
        styleLabel->setText(QCoreApplication::translate("SettingsDialog", "Style", nullptr));
        familyLabel->setText(QCoreApplication::translate("SettingsDialog", "Family", nullptr));
        resetFontButton->setText(QCoreApplication::translate("SettingsDialog", "Reset", nullptr));
        applyFontButton->setText(QCoreApplication::translate("SettingsDialog", "Apply", nullptr));
        settingsTab->setTabText(settingsTab->indexOf(fontTab), QCoreApplication::translate("SettingsDialog", "Font", nullptr));
        addHostButton->setText(QCoreApplication::translate("SettingsDialog", "Add", nullptr));
        removeHostButton->setText(QCoreApplication::translate("SettingsDialog", "Remove", nullptr));
        hostButton->setText(QCoreApplication::translate("SettingsDialog", "...", nullptr));
        savedHostsGroupBox->setTitle(QCoreApplication::translate("SettingsDialog", "Saved Hosts", nullptr));
        settingsTab->setTabText(settingsTab->indexOf(tab), QCoreApplication::translate("SettingsDialog", "Hosts", nullptr));
        startupToolbarCheckBox->setText(QCoreApplication::translate("SettingsDialog", "Show Toolbar on Startup", nullptr));
        nativeToolbarCheckBox->setText(QCoreApplication::translate("SettingsDialog", "Native Toolbar Style", nullptr));
        toolbarLocationLabel->setText(QCoreApplication::translate("SettingsDialog", "Location and Orientation:", nullptr));
        toolbarAreasComboBox->setItemText(0, QCoreApplication::translate("SettingsDialog", "Top and Horizonal", nullptr));
        toolbarAreasComboBox->setItemText(1, QCoreApplication::translate("SettingsDialog", "Right and Vertical", nullptr));

        toolbarActionsLabel->setText(QCoreApplication::translate("SettingsDialog", "Actions:", nullptr));
        settingsTab->setTabText(settingsTab->indexOf(toolbarTab), QCoreApplication::translate("SettingsDialog", "Toolbar", nullptr));
        closeSettingsButton->setText(QCoreApplication::translate("SettingsDialog", "&Close", nullptr));
#if QT_CONFIG(shortcut)
        closeSettingsButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class SettingsDialog: public Ui_SettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSDIALOG_H
