/*
 * Performance Metrics Name Space Map
 * Built by pmgenmap.sh from the file
 * ifpropmetrics.map
 * on Wed May 27 20:01:55 UTC 2026
 *
 * Do not edit this file!
 */

/*
 *
 * metric names and access macros for ifprop.[ch]
 *
 */

const char *ifpropmetrics[] = {
#define IF_SPEED	0
	"network.interface.baudrate",
#define IF_DUPLEX	1
	"network.interface.duplex",
#define IF_TYPE	2
	"network.interface.type",
#define IF_VIRTUAL	3
	"network.interface.virtual",
#define IF_WIRELESS	4
	"network.interface.wireless",

/* end marker - provides count */
#define IF_NMETRICS	5
	".",

};

