/********************************************************************************
** Form generated from reading UI file 'aboutdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QLabel *aboutPixmapLabel;
    QLabel *versionTextLabel;
    QSpacerItem *spacerItem1;
    QLabel *authorsTextLabel;
    QSpacerItem *spacerItem2;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem3;
    QPushButton *aboutOKButton;
    QSpacerItem *spacerItem4;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName(QString::fromUtf8("AboutDialog"));
        AboutDialog->resize(330, 240);
        AboutDialog->setMinimumSize(QSize(330, 240));
        AboutDialog->setMaximumSize(QSize(330, 260));
        const QIcon icon = QIcon(QString::fromUtf8(":/images/pmchart.png"));
        AboutDialog->setWindowIcon(icon);
        AboutDialog->setModal(true);
        gridLayout = new QGridLayout(AboutDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        aboutPixmapLabel = new QLabel(AboutDialog);
        aboutPixmapLabel->setObjectName(QString::fromUtf8("aboutPixmapLabel"));
        aboutPixmapLabel->setPixmap(QPixmap(QString::fromUtf8(":/images/pmchart.png")));
        aboutPixmapLabel->setScaledContents(false);
        aboutPixmapLabel->setWordWrap(false);

        hboxLayout->addWidget(aboutPixmapLabel);

        versionTextLabel = new QLabel(AboutDialog);
        versionTextLabel->setObjectName(QString::fromUtf8("versionTextLabel"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(versionTextLabel->sizePolicy().hasHeightForWidth());
        versionTextLabel->setSizePolicy(sizePolicy);
        versionTextLabel->setAlignment(Qt::AlignCenter);
        versionTextLabel->setWordWrap(true);

        hboxLayout->addWidget(versionTextLabel);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);


        vboxLayout->addLayout(hboxLayout);

        authorsTextLabel = new QLabel(AboutDialog);
        authorsTextLabel->setObjectName(QString::fromUtf8("authorsTextLabel"));
        authorsTextLabel->setMidLineWidth(0);
        authorsTextLabel->setAlignment(Qt::AlignCenter);
        authorsTextLabel->setWordWrap(true);

        vboxLayout->addWidget(authorsTextLabel);

        spacerItem2 = new QSpacerItem(20, 5, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout->addItem(spacerItem2);

        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem3);

        aboutOKButton = new QPushButton(AboutDialog);
        aboutOKButton->setObjectName(QString::fromUtf8("aboutOKButton"));
        aboutOKButton->setFocusPolicy(Qt::TabFocus);

        hboxLayout1->addWidget(aboutOKButton);

        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem4);


        vboxLayout->addLayout(hboxLayout1);


        gridLayout->addLayout(vboxLayout, 0, 0, 1, 1);


        retranslateUi(AboutDialog);

        aboutOKButton->setDefault(true);


        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "About", nullptr));
        versionTextLabel->setText(QCoreApplication::translate("AboutDialog", "<b><i>pmchart</i></b><br>\n"
"<b><i>Version VERSION</i></b>", nullptr));
        authorsTextLabel->setText(QCoreApplication::translate("AboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Copyright 2012, Red Hat.<br />Copyright 2012-2014, Nathan Scott.<br />Copyright 2006, Ken McDonell.<br />Copyright 2006-2010, Aconex.<br />All rights reserved.<br /><span style=\" font-style:italic;\"><br />This program is licensed under the<br />GNU General Public License.<br /></span></p></body></html>", nullptr));
        aboutOKButton->setText(QCoreApplication::translate("AboutDialog", "OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
