/********************************************************************************
** Form generated from reading UI file 'infodialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INFODIALOG_H
#define UI_INFODIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_InfoDialog
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QHBoxLayout *hboxLayout;
    QSpacerItem *Horizontal_Spacing2;
    QPushButton *buttonOk;
    QTabWidget *infoTab;
    QWidget *pminfoTab;
    QGridLayout *gridLayout2;
    QTextEdit *pminfoTextEdit;
    QWidget *pmvalTab;
    QGridLayout *gridLayout3;
    QTextEdit *pmvalTextEdit;

    void setupUi(QDialog *InfoDialog)
    {
        if (InfoDialog->objectName().isEmpty())
            InfoDialog->setObjectName(QString::fromUtf8("InfoDialog"));
        InfoDialog->resize(413, 330);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/help-browser.png"));
        InfoDialog->setWindowIcon(icon);
        InfoDialog->setSizeGripEnabled(true);
        gridLayout = new QGridLayout(InfoDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout1 = new QGridLayout();
#ifndef Q_OS_MAC
        gridLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        Horizontal_Spacing2 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(Horizontal_Spacing2);

        buttonOk = new QPushButton(InfoDialog);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout->addWidget(buttonOk);


        gridLayout1->addLayout(hboxLayout, 1, 0, 1, 1);

        infoTab = new QTabWidget(InfoDialog);
        infoTab->setObjectName(QString::fromUtf8("infoTab"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(infoTab->sizePolicy().hasHeightForWidth());
        infoTab->setSizePolicy(sizePolicy);
        infoTab->setMinimumSize(QSize(250, 150));
        pminfoTab = new QWidget();
        pminfoTab->setObjectName(QString::fromUtf8("pminfoTab"));
        gridLayout2 = new QGridLayout(pminfoTab);
        gridLayout2->setContentsMargins(0, 0, 0, 0);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        pminfoTextEdit = new QTextEdit(pminfoTab);
        pminfoTextEdit->setObjectName(QString::fromUtf8("pminfoTextEdit"));
        QFont font;
        font.setFamily(QString::fromUtf8("Monospace"));
        font.setPointSize(9);
        pminfoTextEdit->setFont(font);
        pminfoTextEdit->setFrameShape(QFrame::Box);
        pminfoTextEdit->setFrameShadow(QFrame::Sunken);
        pminfoTextEdit->setLineWidth(1);
        pminfoTextEdit->setUndoRedoEnabled(false);
        pminfoTextEdit->setReadOnly(true);

        gridLayout2->addWidget(pminfoTextEdit, 0, 0, 1, 1);

        infoTab->addTab(pminfoTab, QString());
        pmvalTab = new QWidget();
        pmvalTab->setObjectName(QString::fromUtf8("pmvalTab"));
        gridLayout3 = new QGridLayout(pmvalTab);
        gridLayout3->setContentsMargins(0, 0, 0, 0);
        gridLayout3->setObjectName(QString::fromUtf8("gridLayout3"));
        pmvalTextEdit = new QTextEdit(pmvalTab);
        pmvalTextEdit->setObjectName(QString::fromUtf8("pmvalTextEdit"));
        pmvalTextEdit->setFont(font);
        pmvalTextEdit->setFrameShape(QFrame::Box);
        pmvalTextEdit->setFrameShadow(QFrame::Sunken);
        pmvalTextEdit->setLineWidth(1);
        pmvalTextEdit->setUndoRedoEnabled(false);
        pmvalTextEdit->setReadOnly(true);

        gridLayout3->addWidget(pmvalTextEdit, 0, 0, 1, 1);

        infoTab->addTab(pmvalTab, QString());

        gridLayout1->addWidget(infoTab, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 1);


        retranslateUi(InfoDialog);

        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(InfoDialog);
    } // setupUi

    void retranslateUi(QDialog *InfoDialog)
    {
        InfoDialog->setWindowTitle(QCoreApplication::translate("InfoDialog", "Metric Information", nullptr));
        buttonOk->setText(QCoreApplication::translate("InfoDialog", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        pminfoTextEdit->setText(QString());
        infoTab->setTabText(infoTab->indexOf(pminfoTab), QCoreApplication::translate("InfoDialog", "pminfo", nullptr));
        infoTab->setTabText(infoTab->indexOf(pmvalTab), QCoreApplication::translate("InfoDialog", "pmval", nullptr));
    } // retranslateUi

};

namespace Ui {
    class InfoDialog: public Ui_InfoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_INFODIALOG_H
