/********************************************************************************
** Form generated from reading UI file 'tabdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TABDIALOG_H
#define UI_TABDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_TabDialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QLabel *tabSourceLabel;
    QSpacerItem *spacerItem;
    QRadioButton *liveHostRadioButton;
    QRadioButton *archivesRadioButton;
    QSpacerItem *spacerItem1;
    QHBoxLayout *hboxLayout1;
    QLabel *tabTextLabel;
    QLineEdit *labelLineEdit;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacerItem2;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QDialog *TabDialog)
    {
        if (TabDialog->objectName().isEmpty())
            TabDialog->setObjectName(QString::fromUtf8("TabDialog"));
        TabDialog->setWindowModality(Qt::WindowModal);
        TabDialog->resize(340, 150);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TabDialog->sizePolicy().hasHeightForWidth());
        TabDialog->setSizePolicy(sizePolicy);
        TabDialog->setMinimumSize(QSize(340, 150));
        TabDialog->setMaximumSize(QSize(350, 250));
        TabDialog->setFocusPolicy(Qt::WheelFocus);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/tab-edit.png"));
        TabDialog->setWindowIcon(icon);
        TabDialog->setSizeGripEnabled(false);
        gridLayout = new QGridLayout(TabDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        tabSourceLabel = new QLabel(TabDialog);
        tabSourceLabel->setObjectName(QString::fromUtf8("tabSourceLabel"));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(tabSourceLabel->sizePolicy().hasHeightForWidth());
        tabSourceLabel->setSizePolicy(sizePolicy1);
        tabSourceLabel->setMinimumSize(QSize(50, 0));

        hboxLayout->addWidget(tabSourceLabel);

        spacerItem = new QSpacerItem(16, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        liveHostRadioButton = new QRadioButton(TabDialog);
        liveHostRadioButton->setObjectName(QString::fromUtf8("liveHostRadioButton"));
        sizePolicy.setHeightForWidth(liveHostRadioButton->sizePolicy().hasHeightForWidth());
        liveHostRadioButton->setSizePolicy(sizePolicy);
        liveHostRadioButton->setMinimumSize(QSize(100, 0));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/computer.png"), QSize(), QIcon::Normal, QIcon::Off);
        liveHostRadioButton->setIcon(icon1);
        liveHostRadioButton->setChecked(true);
        liveHostRadioButton->setAutoExclusive(false);

        hboxLayout->addWidget(liveHostRadioButton);

        archivesRadioButton = new QRadioButton(TabDialog);
        archivesRadioButton->setObjectName(QString::fromUtf8("archivesRadioButton"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/archive.png"), QSize(), QIcon::Normal, QIcon::Off);
        archivesRadioButton->setIcon(icon2);
        archivesRadioButton->setAutoExclusive(false);

        hboxLayout->addWidget(archivesRadioButton);

        spacerItem1 = new QSpacerItem(34, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        tabTextLabel = new QLabel(TabDialog);
        tabTextLabel->setObjectName(QString::fromUtf8("tabTextLabel"));
        tabTextLabel->setWordWrap(false);

        hboxLayout1->addWidget(tabTextLabel);

        labelLineEdit = new QLineEdit(TabDialog);
        labelLineEdit->setObjectName(QString::fromUtf8("labelLineEdit"));

        hboxLayout1->addWidget(labelLineEdit);


        gridLayout->addLayout(hboxLayout1, 1, 0, 1, 1);

        verticalSpacer = new QSpacerItem(5, 5, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        spacerItem2 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem2);

        buttonOk = new QPushButton(TabDialog);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout2->addWidget(buttonOk);

        buttonCancel = new QPushButton(TabDialog);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout2->addWidget(buttonCancel);


        gridLayout->addLayout(hboxLayout2, 3, 0, 1, 1);


        retranslateUi(TabDialog);

        buttonOk->setDefault(true);
        buttonCancel->setDefault(false);


        QMetaObject::connectSlotsByName(TabDialog);
    } // setupUi

    void retranslateUi(QDialog *TabDialog)
    {
        TabDialog->setWindowTitle(QCoreApplication::translate("TabDialog", "Add Tab", nullptr));
        tabSourceLabel->setText(QCoreApplication::translate("TabDialog", "Source:", nullptr));
        liveHostRadioButton->setText(QCoreApplication::translate("TabDialog", "Live", nullptr));
        archivesRadioButton->setText(QCoreApplication::translate("TabDialog", "Archives", nullptr));
        tabTextLabel->setText(QCoreApplication::translate("TabDialog", "Label:", nullptr));
        buttonOk->setText(QCoreApplication::translate("TabDialog", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        buttonCancel->setText(QCoreApplication::translate("TabDialog", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class TabDialog: public Ui_TabDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TABDIALOG_H
