/********************************************************************************
** Form generated from reading UI file 'seealsodialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SEEALSODIALOG_H
#define UI_SEEALSODIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SeeAlsoDialog
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QLabel *seeAlsoPCPPixmapLabel;
    QSpacerItem *spacerItem;
    QVBoxLayout *vboxLayout2;
    QLabel *versionPCPTextLabel;
    QLabel *versionQwtTextLabel;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem1;
    QPushButton *seeAlsoOKButton;
    QSpacerItem *spacerItem2;

    void setupUi(QDialog *SeeAlsoDialog)
    {
        if (SeeAlsoDialog->objectName().isEmpty())
            SeeAlsoDialog->setObjectName(QString::fromUtf8("SeeAlsoDialog"));
        SeeAlsoDialog->resize(410, 250);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SeeAlsoDialog->sizePolicy().hasHeightForWidth());
        SeeAlsoDialog->setSizePolicy(sizePolicy);
        SeeAlsoDialog->setMinimumSize(QSize(410, 250));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/pmtime.png"), QSize(), QIcon::Normal, QIcon::Off);
        SeeAlsoDialog->setWindowIcon(icon);
        gridLayout = new QGridLayout(SeeAlsoDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        vboxLayout = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        vboxLayout1 = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        seeAlsoPCPPixmapLabel = new QLabel(SeeAlsoDialog);
        seeAlsoPCPPixmapLabel->setObjectName(QString::fromUtf8("seeAlsoPCPPixmapLabel"));
        sizePolicy.setHeightForWidth(seeAlsoPCPPixmapLabel->sizePolicy().hasHeightForWidth());
        seeAlsoPCPPixmapLabel->setSizePolicy(sizePolicy);
        seeAlsoPCPPixmapLabel->setMinimumSize(QSize(76, 82));
        seeAlsoPCPPixmapLabel->setPixmap(QPixmap(QString::fromUtf8(":/images/aboutpcp.png")));
        seeAlsoPCPPixmapLabel->setScaledContents(false);
        seeAlsoPCPPixmapLabel->setWordWrap(false);

        vboxLayout1->addWidget(seeAlsoPCPPixmapLabel);

        spacerItem = new QSpacerItem(20, 51, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout1->addItem(spacerItem);


        hboxLayout->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
#ifndef Q_OS_MAC
        vboxLayout2->setSpacing(6);
#endif
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        versionPCPTextLabel = new QLabel(SeeAlsoDialog);
        versionPCPTextLabel->setObjectName(QString::fromUtf8("versionPCPTextLabel"));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(versionPCPTextLabel->sizePolicy().hasHeightForWidth());
        versionPCPTextLabel->setSizePolicy(sizePolicy1);
        versionPCPTextLabel->setAlignment(Qt::AlignCenter);
        versionPCPTextLabel->setWordWrap(true);

        vboxLayout2->addWidget(versionPCPTextLabel);

        versionQwtTextLabel = new QLabel(SeeAlsoDialog);
        versionQwtTextLabel->setObjectName(QString::fromUtf8("versionQwtTextLabel"));
        sizePolicy1.setHeightForWidth(versionQwtTextLabel->sizePolicy().hasHeightForWidth());
        versionQwtTextLabel->setSizePolicy(sizePolicy1);
        versionQwtTextLabel->setAlignment(Qt::AlignCenter);
        versionQwtTextLabel->setWordWrap(true);

        vboxLayout2->addWidget(versionQwtTextLabel);


        hboxLayout->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem1);

        seeAlsoOKButton = new QPushButton(SeeAlsoDialog);
        seeAlsoOKButton->setObjectName(QString::fromUtf8("seeAlsoOKButton"));
        seeAlsoOKButton->setFocusPolicy(Qt::TabFocus);

        hboxLayout1->addWidget(seeAlsoOKButton);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem2);


        vboxLayout->addLayout(hboxLayout1);


        gridLayout->addLayout(vboxLayout, 0, 0, 1, 1);


        retranslateUi(SeeAlsoDialog);

        seeAlsoOKButton->setDefault(true);


        QMetaObject::connectSlotsByName(SeeAlsoDialog);
    } // setupUi

    void retranslateUi(QDialog *SeeAlsoDialog)
    {
        SeeAlsoDialog->setWindowTitle(QCoreApplication::translate("SeeAlsoDialog", "Acknowledgements", nullptr));
        versionPCPTextLabel->setText(QCoreApplication::translate("SeeAlsoDialog", "<b><i>Performance Co-Pilot (PCP)</i></b><br>\n"
"Copyright (c) 2010-2018 Red Hat<br>\n"
"Copyright (c) 2006-2012 Aconex<br>\n"
"Copyright (c) 1995-2007 SGI<br>\n"
"... and many, many others!<br>\n"
"<i>https://pcp.io</i>", nullptr));
        versionQwtTextLabel->setText(QCoreApplication::translate("SeeAlsoDialog", "<b><i>Qt Widgets for Technical Apps (Qwt)</i></b><br>\n"
"Copyright (c) 1997   Josef Wilgen<br>\n"
"Copyright (c) 2002   Uwe Rathmann<br>\n"
"<i>http://qwt.sourceforge.net/</i>", nullptr));
        seeAlsoOKButton->setText(QCoreApplication::translate("SeeAlsoDialog", "OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SeeAlsoDialog: public Ui_SeeAlsoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SEEALSODIALOG_H
