/****************************************************************************
** Meta object code from reading C++ file 'chartdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "chartdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'chartdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ChartDialog_t {
    QByteArrayData data[40];
    char stringdata0[788];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ChartDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ChartDialog_t qt_meta_stringdata_ChartDialog = {
    {
QT_MOC_LITERAL(0, 0, 11), // "ChartDialog"
QT_MOC_LITERAL(1, 12, 6), // "newCol"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 4), // "QRgb"
QT_MOC_LITERAL(4, 25, 16), // "buttonOk_clicked"
QT_MOC_LITERAL(5, 42, 19), // "buttonApply_clicked"
QT_MOC_LITERAL(6, 62, 32), // "chartMetricsItemSelectionChanged"
QT_MOC_LITERAL(7, 95, 36), // "availableMetricsItemSelection..."
QT_MOC_LITERAL(8, 132, 28), // "availableMetricsItemExpanded"
QT_MOC_LITERAL(9, 161, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(10, 178, 40), // "availableMetricsTreeWidget_do..."
QT_MOC_LITERAL(11, 219, 11), // "QModelIndex"
QT_MOC_LITERAL(12, 231, 23), // "metricInfoButtonClicked"
QT_MOC_LITERAL(13, 255, 25), // "metricSearchButtonClicked"
QT_MOC_LITERAL(14, 281, 25), // "metricDeleteButtonClicked"
QT_MOC_LITERAL(15, 307, 22), // "metricAddButtonClicked"
QT_MOC_LITERAL(16, 330, 20), // "archiveButtonClicked"
QT_MOC_LITERAL(17, 351, 17), // "hostButtonClicked"
QT_MOC_LITERAL(18, 369, 19), // "sourceButtonClicked"
QT_MOC_LITERAL(19, 389, 15), // "legendOnClicked"
QT_MOC_LITERAL(20, 405, 16), // "legendOffClicked"
QT_MOC_LITERAL(21, 422, 21), // "antiAliasingOnClicked"
QT_MOC_LITERAL(22, 444, 22), // "antiAliasingOffClicked"
QT_MOC_LITERAL(23, 467, 18), // "autoScaleOnClicked"
QT_MOC_LITERAL(24, 486, 19), // "autoScaleOffClicked"
QT_MOC_LITERAL(25, 506, 18), // "rateConvertClicked"
QT_MOC_LITERAL(26, 525, 23), // "antiAliasingAutoClicked"
QT_MOC_LITERAL(27, 549, 24), // "yAxisMinimumValueChanged"
QT_MOC_LITERAL(28, 574, 24), // "yAxisMaximumValueChanged"
QT_MOC_LITERAL(29, 599, 8), // "newColor"
QT_MOC_LITERAL(30, 608, 15), // "newColorTypedIn"
QT_MOC_LITERAL(31, 624, 23), // "applyColorButtonClicked"
QT_MOC_LITERAL(32, 648, 24), // "revertColorButtonClicked"
QT_MOC_LITERAL(33, 673, 6), // "newHsv"
QT_MOC_LITERAL(34, 680, 6), // "setRgb"
QT_MOC_LITERAL(35, 687, 5), // "rgbEd"
QT_MOC_LITERAL(36, 693, 5), // "hsvEd"
QT_MOC_LITERAL(37, 699, 33), // "plotLabelLineEdit_editingFini..."
QT_MOC_LITERAL(38, 733, 39), // "colorSchemeComboBox_currentIn..."
QT_MOC_LITERAL(39, 773, 14) // "languageChange"

    },
    "ChartDialog\0newCol\0\0QRgb\0buttonOk_clicked\0"
    "buttonApply_clicked\0"
    "chartMetricsItemSelectionChanged\0"
    "availableMetricsItemSelectionChanged\0"
    "availableMetricsItemExpanded\0"
    "QTreeWidgetItem*\0"
    "availableMetricsTreeWidget_doubleClicked\0"
    "QModelIndex\0metricInfoButtonClicked\0"
    "metricSearchButtonClicked\0"
    "metricDeleteButtonClicked\0"
    "metricAddButtonClicked\0archiveButtonClicked\0"
    "hostButtonClicked\0sourceButtonClicked\0"
    "legendOnClicked\0legendOffClicked\0"
    "antiAliasingOnClicked\0antiAliasingOffClicked\0"
    "autoScaleOnClicked\0autoScaleOffClicked\0"
    "rateConvertClicked\0antiAliasingAutoClicked\0"
    "yAxisMinimumValueChanged\0"
    "yAxisMaximumValueChanged\0newColor\0"
    "newColorTypedIn\0applyColorButtonClicked\0"
    "revertColorButtonClicked\0newHsv\0setRgb\0"
    "rgbEd\0hsvEd\0plotLabelLineEdit_editingFinished\0"
    "colorSchemeComboBox_currentIndexChanged\0"
    "languageChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ChartDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      35,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  189,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  192,    2, 0x0a /* Public */,
       5,    0,  193,    2, 0x0a /* Public */,
       6,    0,  194,    2, 0x0a /* Public */,
       7,    0,  195,    2, 0x0a /* Public */,
       8,    1,  196,    2, 0x0a /* Public */,
      10,    1,  199,    2, 0x0a /* Public */,
      12,    0,  202,    2, 0x0a /* Public */,
      13,    0,  203,    2, 0x0a /* Public */,
      14,    0,  204,    2, 0x0a /* Public */,
      15,    0,  205,    2, 0x0a /* Public */,
      16,    0,  206,    2, 0x0a /* Public */,
      17,    0,  207,    2, 0x0a /* Public */,
      18,    0,  208,    2, 0x0a /* Public */,
      19,    0,  209,    2, 0x0a /* Public */,
      20,    0,  210,    2, 0x0a /* Public */,
      21,    0,  211,    2, 0x0a /* Public */,
      22,    0,  212,    2, 0x0a /* Public */,
      23,    0,  213,    2, 0x0a /* Public */,
      24,    0,  214,    2, 0x0a /* Public */,
      25,    0,  215,    2, 0x0a /* Public */,
      26,    0,  216,    2, 0x0a /* Public */,
      27,    1,  217,    2, 0x0a /* Public */,
      28,    1,  220,    2, 0x0a /* Public */,
      29,    1,  223,    2, 0x0a /* Public */,
      30,    1,  226,    2, 0x0a /* Public */,
      31,    0,  229,    2, 0x0a /* Public */,
      32,    0,  230,    2, 0x0a /* Public */,
      33,    3,  231,    2, 0x0a /* Public */,
      34,    1,  238,    2, 0x0a /* Public */,
      35,    0,  241,    2, 0x0a /* Public */,
      36,    0,  242,    2, 0x0a /* Public */,
      37,    0,  243,    2, 0x0a /* Public */,
      38,    1,  244,    2, 0x0a /* Public */,
      39,    0,  247,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::QColor,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

       0        // eod
};

void ChartDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChartDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->newCol((*reinterpret_cast< QRgb(*)>(_a[1]))); break;
        case 1: _t->buttonOk_clicked(); break;
        case 2: _t->buttonApply_clicked(); break;
        case 3: _t->chartMetricsItemSelectionChanged(); break;
        case 4: _t->availableMetricsItemSelectionChanged(); break;
        case 5: _t->availableMetricsItemExpanded((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 6: _t->availableMetricsTreeWidget_doubleClicked((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 7: _t->metricInfoButtonClicked(); break;
        case 8: _t->metricSearchButtonClicked(); break;
        case 9: _t->metricDeleteButtonClicked(); break;
        case 10: _t->metricAddButtonClicked(); break;
        case 11: _t->archiveButtonClicked(); break;
        case 12: _t->hostButtonClicked(); break;
        case 13: _t->sourceButtonClicked(); break;
        case 14: _t->legendOnClicked(); break;
        case 15: _t->legendOffClicked(); break;
        case 16: _t->antiAliasingOnClicked(); break;
        case 17: _t->antiAliasingOffClicked(); break;
        case 18: _t->autoScaleOnClicked(); break;
        case 19: _t->autoScaleOffClicked(); break;
        case 20: _t->rateConvertClicked(); break;
        case 21: _t->antiAliasingAutoClicked(); break;
        case 22: _t->yAxisMinimumValueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 23: _t->yAxisMaximumValueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 24: _t->newColor((*reinterpret_cast< QColor(*)>(_a[1]))); break;
        case 25: _t->newColorTypedIn((*reinterpret_cast< QRgb(*)>(_a[1]))); break;
        case 26: _t->applyColorButtonClicked(); break;
        case 27: _t->revertColorButtonClicked(); break;
        case 28: _t->newHsv((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 29: _t->setRgb((*reinterpret_cast< QRgb(*)>(_a[1]))); break;
        case 30: _t->rgbEd(); break;
        case 31: _t->hsvEd(); break;
        case 32: _t->plotLabelLineEdit_editingFinished(); break;
        case 33: _t->colorSchemeComboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->languageChange(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ChartDialog::*)(QRgb );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChartDialog::newCol)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ChartDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ChartDialog.data,
    qt_meta_data_ChartDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ChartDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ChartDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ChartDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::ChartDialog"))
        return static_cast< Ui::ChartDialog*>(this);
    return QDialog::qt_metacast(_clname);
}

int ChartDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void ChartDialog::newCol(QRgb _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
