/*
 * Performance Metrics Name Space Map
 * Built by pmgenmap.sh from the file
 * netprocmetrics.map
 * on Wed May 27 20:01:55 UTC 2026
 *
 * Do not edit this file!
 */

/*
 *
 * metric names and access macros for netproc.c
 *
 */
const char *netbpfprocmetrics[] = {
#define TASK_NETBPF_TCPSND	0
	"bpf.proc.net.tcp.send.packets",
#define TASK_NETBPF_TCPSSZ	1
	"bpf.proc.net.tcp.send.bytes",
#define TASK_NETBPF_TCPRCV	2
	"bpf.proc.net.tcp.recv.packets",
#define TASK_NETBPF_TCPRSZ	3
	"bpf.proc.net.tcp.recv.bytes",

#define TASK_NETBPF_UDPSND	4
	"bpf.proc.net.udp.send.packets",
#define TASK_NETBPF_UDPSSZ	5
	"bpf.proc.net.udp.send.bytes",
#define TASK_NETBPF_UDPRCV	6
	"bpf.proc.net.udp.recv.packets",
#define TASK_NETBPF_UDPRSZ	7
	"bpf.proc.net.udp.recv.bytes",

/* end marker - provides count */
#define TASK_NETBPF_NMETRICS	8
	".",

};

