/********************************************************************************
** Form generated from reading UI file 'pmtimelive.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PMTIMELIVE_H
#define UI_PMTIMELIVE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "console.h"
#include "pmtime.h"
#include "timezone.h"

QT_BEGIN_NAMESPACE

class Ui_PmTimeLive
{
public:
    QAction *helpAboutAction;
    QAction *fileHideAction;
    QAction *optionsDetailShow_MillisecondsAction;
    QAction *optionsDetailShow_YearAction;
    QAction *optionsShowConsoleAction;
    QAction *helpAboutQtAction;
    QAction *helpSeeAlsoAction;
    QAction *helpWhats_ThisAction;
    QAction *helpManualAction;
    QWidget *widget;
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QLabel *textLabel1;
    QLabel *textLabel2;
    QLabel *textLabel3;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QLineEdit *lineEditDelta;
    QComboBox *comboBoxDelta;
    QLineEdit *lineEditCtime;
    QHBoxLayout *hboxLayout2;
    QToolButton *buttonStop;
    QToolButton *buttonPlay;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;
    QMenuBar *MenuBar;
    QMenu *Help;
    QMenu *File;
    QMenu *Options;
    QMenu *optionsDetailAction;
    QMenu *optionsTimezoneAction;

    void setupUi(QMainWindow *PmTimeLive)
    {
        if (PmTimeLive->objectName().isEmpty())
            PmTimeLive->setObjectName(QString::fromUtf8("PmTimeLive"));
        PmTimeLive->resize(310, 155);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(PmTimeLive->sizePolicy().hasHeightForWidth());
        PmTimeLive->setSizePolicy(sizePolicy);
        PmTimeLive->setMinimumSize(QSize(310, 155));
        PmTimeLive->setMaximumSize(QSize(310, 155));
        const QIcon icon = QIcon(QString::fromUtf8(":/images/pmtime.png"));
        PmTimeLive->setWindowIcon(icon);
        helpAboutAction = new QAction(PmTimeLive);
        helpAboutAction->setObjectName(QString::fromUtf8("helpAboutAction"));
        helpAboutAction->setIcon(icon);
        fileHideAction = new QAction(PmTimeLive);
        fileHideAction->setObjectName(QString::fromUtf8("fileHideAction"));
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/edit-clear.png"));
        fileHideAction->setIcon(icon1);
        optionsDetailShow_MillisecondsAction = new QAction(PmTimeLive);
        optionsDetailShow_MillisecondsAction->setObjectName(QString::fromUtf8("optionsDetailShow_MillisecondsAction"));
        optionsDetailShow_MillisecondsAction->setCheckable(true);
        optionsDetailShow_YearAction = new QAction(PmTimeLive);
        optionsDetailShow_YearAction->setObjectName(QString::fromUtf8("optionsDetailShow_YearAction"));
        optionsDetailShow_YearAction->setCheckable(true);
        optionsShowConsoleAction = new QAction(PmTimeLive);
        optionsShowConsoleAction->setObjectName(QString::fromUtf8("optionsShowConsoleAction"));
        helpAboutQtAction = new QAction(PmTimeLive);
        helpAboutQtAction->setObjectName(QString::fromUtf8("helpAboutQtAction"));
        helpSeeAlsoAction = new QAction(PmTimeLive);
        helpSeeAlsoAction->setObjectName(QString::fromUtf8("helpSeeAlsoAction"));
        helpWhats_ThisAction = new QAction(PmTimeLive);
        helpWhats_ThisAction->setObjectName(QString::fromUtf8("helpWhats_ThisAction"));
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/whatsthis.png"));
        helpWhats_ThisAction->setIcon(icon2);
        helpManualAction = new QAction(PmTimeLive);
        helpManualAction->setObjectName(QString::fromUtf8("helpManualAction"));
        widget = new QWidget(PmTimeLive);
        widget->setObjectName(QString::fromUtf8("widget"));
        gridLayout = new QGridLayout(widget);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(9, 9, 9, 9);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        textLabel1 = new QLabel(widget);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel1->setWordWrap(false);

        vboxLayout->addWidget(textLabel1);

        textLabel2 = new QLabel(widget);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));
        textLabel2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel2->setWordWrap(false);

        vboxLayout->addWidget(textLabel2);

        textLabel3 = new QLabel(widget);
        textLabel3->setObjectName(QString::fromUtf8("textLabel3"));
        textLabel3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        textLabel3->setWordWrap(false);

        vboxLayout->addWidget(textLabel3);


        hboxLayout->addLayout(vboxLayout);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        lineEditDelta = new QLineEdit(widget);
        lineEditDelta->setObjectName(QString::fromUtf8("lineEditDelta"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lineEditDelta->sizePolicy().hasHeightForWidth());
        lineEditDelta->setSizePolicy(sizePolicy1);
        lineEditDelta->setMinimumSize(QSize(98, 0));
        lineEditDelta->setMaximumSize(QSize(98, 32767));

        hboxLayout1->addWidget(lineEditDelta);

        comboBoxDelta = new QComboBox(widget);
        comboBoxDelta->addItem(QString());
        comboBoxDelta->addItem(QString());
        comboBoxDelta->addItem(QString());
        comboBoxDelta->addItem(QString());
        comboBoxDelta->addItem(QString());
        comboBoxDelta->addItem(QString());
        comboBoxDelta->setObjectName(QString::fromUtf8("comboBoxDelta"));

        hboxLayout1->addWidget(comboBoxDelta);


        vboxLayout1->addLayout(hboxLayout1);

        lineEditCtime = new QLineEdit(widget);
        lineEditCtime->setObjectName(QString::fromUtf8("lineEditCtime"));
        sizePolicy1.setHeightForWidth(lineEditCtime->sizePolicy().hasHeightForWidth());
        lineEditCtime->setSizePolicy(sizePolicy1);
        lineEditCtime->setMinimumSize(QSize(200, 0));
        lineEditCtime->setMaximumSize(QSize(32767, 32767));
        lineEditCtime->setAcceptDrops(false);
        lineEditCtime->setReadOnly(true);

        vboxLayout1->addWidget(lineEditCtime);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        buttonStop = new QToolButton(widget);
        buttonStop->setObjectName(QString::fromUtf8("buttonStop"));
        sizePolicy.setHeightForWidth(buttonStop->sizePolicy().hasHeightForWidth());
        buttonStop->setSizePolicy(sizePolicy);
        buttonStop->setMinimumSize(QSize(54, 36));
        buttonStop->setIconSize(QSize(52, 32));

        hboxLayout2->addWidget(buttonStop);

        buttonPlay = new QToolButton(widget);
        buttonPlay->setObjectName(QString::fromUtf8("buttonPlay"));
        sizePolicy.setHeightForWidth(buttonPlay->sizePolicy().hasHeightForWidth());
        buttonPlay->setSizePolicy(sizePolicy);
        buttonPlay->setMinimumSize(QSize(54, 36));
        buttonPlay->setIconSize(QSize(52, 32));

        hboxLayout2->addWidget(buttonPlay);

        spacerItem = new QSpacerItem(29, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem);


        vboxLayout1->addLayout(hboxLayout2);


        hboxLayout->addLayout(vboxLayout1);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        spacerItem1 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        gridLayout->addItem(spacerItem1, 1, 0, 1, 1);

        PmTimeLive->setCentralWidget(widget);
        MenuBar = new QMenuBar(PmTimeLive);
        MenuBar->setObjectName(QString::fromUtf8("MenuBar"));
        MenuBar->setGeometry(QRect(0, 0, 310, 25));
        Help = new QMenu(MenuBar);
        Help->setObjectName(QString::fromUtf8("Help"));
        File = new QMenu(MenuBar);
        File->setObjectName(QString::fromUtf8("File"));
        Options = new QMenu(MenuBar);
        Options->setObjectName(QString::fromUtf8("Options"));
        optionsDetailAction = new QMenu(Options);
        optionsDetailAction->setObjectName(QString::fromUtf8("optionsDetailAction"));
        optionsTimezoneAction = new QMenu(Options);
        optionsTimezoneAction->setObjectName(QString::fromUtf8("optionsTimezoneAction"));
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/internet-web-browser.png"));
        optionsTimezoneAction->setWindowIcon(icon3);
        optionsTimezoneAction->setIcon(icon3);
        PmTimeLive->setMenuBar(MenuBar);

        MenuBar->addAction(File->menuAction());
        MenuBar->addAction(Options->menuAction());
        MenuBar->addSeparator();
        MenuBar->addAction(Help->menuAction());
        Help->addAction(helpManualAction);
        Help->addSeparator();
        Help->addAction(helpAboutAction);
        Help->addAction(helpAboutQtAction);
        Help->addAction(helpSeeAlsoAction);
        Help->addSeparator();
        Help->addAction(helpWhats_ThisAction);
        File->addAction(fileHideAction);
        Options->addAction(optionsTimezoneAction->menuAction());
        Options->addAction(optionsDetailAction->menuAction());
        Options->addAction(optionsShowConsoleAction);
        optionsDetailAction->addAction(optionsDetailShow_MillisecondsAction);
        optionsDetailAction->addAction(optionsDetailShow_YearAction);

        retranslateUi(PmTimeLive);

        comboBoxDelta->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(PmTimeLive);
    } // setupUi

    void retranslateUi(QMainWindow *PmTimeLive)
    {
        PmTimeLive->setWindowTitle(QCoreApplication::translate("PmTimeLive", "PCP Live Time Control", nullptr));
        PmTimeLive->setWindowIconText(QString());
#if QT_CONFIG(tooltip)
        PmTimeLive->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        PmTimeLive->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Live Time Control</b><br>\n"
"A time server, sending time related information (current time,\n"
"metric value fetch interval, timezone) to one or more client\n"
"programs (e.g. pmchart, pmval, etc).", nullptr));
#endif // QT_CONFIG(whatsthis)
        helpAboutAction->setText(QCoreApplication::translate("PmTimeLive", "&About", nullptr));
        helpAboutAction->setIconText(QCoreApplication::translate("PmTimeLive", "About pmtime", nullptr));
#if QT_CONFIG(whatsthis)
        helpAboutAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Help|About</b><br>\n"
"Click this menu option to bring up the About <i>pmtime</i> dialog with version, author, and licensing information.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        helpAboutAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        fileHideAction->setText(QCoreApplication::translate("PmTimeLive", "&Hide", nullptr));
        fileHideAction->setIconText(QCoreApplication::translate("PmTimeLive", "Hide Live Time Control window", nullptr));
#if QT_CONFIG(tooltip)
        fileHideAction->setToolTip(QCoreApplication::translate("PmTimeLive", "Hide Live Time Control window", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        fileHideAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>File|Hide</b><br>\n"
"Click on this menu option to dismiss the Time Control window,\n"
"for the time being.  As the Time Control serves one or\n"
"more client programs (which are completely reliant on the Time\n"
"Controls), this action does not end the Time Control program.\n"
"Rather, it simply removes the window from the visible set.<br>\n"
"The client programs will typically provide a mechanism for making\n"
"the Time Controls visible.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        fileHideAction->setShortcut(QCoreApplication::translate("PmTimeLive", "Ctrl+H", nullptr));
#endif // QT_CONFIG(shortcut)
        optionsDetailShow_MillisecondsAction->setText(QCoreApplication::translate("PmTimeLive", "Show &Milliseconds", nullptr));
        optionsDetailShow_MillisecondsAction->setIconText(QCoreApplication::translate("PmTimeLive", "Show Milliseconds in Time", nullptr));
#if QT_CONFIG(whatsthis)
        optionsDetailShow_MillisecondsAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Options|Detail|Milliseconds</b><br>\n"
"Display millisecond precision (or not) in the current time interval and\n"
"position.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        optionsDetailShow_MillisecondsAction->setShortcut(QCoreApplication::translate("PmTimeLive", "Ctrl+M", nullptr));
#endif // QT_CONFIG(shortcut)
        optionsDetailShow_YearAction->setText(QCoreApplication::translate("PmTimeLive", "Show &Year", nullptr));
        optionsDetailShow_YearAction->setIconText(QCoreApplication::translate("PmTimeLive", "Show Year in Time", nullptr));
#if QT_CONFIG(whatsthis)
        optionsDetailShow_YearAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Options|Detail|Year</b><br>\n"
"Display the year (or not) in the current time position.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        optionsDetailShow_YearAction->setShortcut(QCoreApplication::translate("PmTimeLive", "Ctrl+Y", nullptr));
#endif // QT_CONFIG(shortcut)
        optionsShowConsoleAction->setText(QCoreApplication::translate("PmTimeLive", "&Show Console", nullptr));
        optionsShowConsoleAction->setIconText(QCoreApplication::translate("PmTimeLive", "Show Diagnostic Console", nullptr));
#if QT_CONFIG(whatsthis)
        optionsShowConsoleAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Option|Show Console</b><br>\n"
"Click here to display the diagnostic console with whatever debug\n"
"information has been sent there.  This menu option is available\n"
"only when the -D command line option is in use, which allows a\n"
"level of runtime verbosity to be configured (default is \"none\").", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        optionsShowConsoleAction->setShortcut(QCoreApplication::translate("PmTimeLive", "Ctrl+C", nullptr));
#endif // QT_CONFIG(shortcut)
        helpAboutQtAction->setText(QCoreApplication::translate("PmTimeLive", "About Qt", nullptr));
        helpAboutQtAction->setIconText(QCoreApplication::translate("PmTimeLive", "About Qt", nullptr));
#if QT_CONFIG(whatsthis)
        helpAboutQtAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Help|About Qt</b><br>\n"
"Click on this menu option to bring up the See Also dialog, with author and website information about Qt - which provides software infrastructure on which this tool was built.", nullptr));
#endif // QT_CONFIG(whatsthis)
        helpSeeAlsoAction->setText(QCoreApplication::translate("PmTimeLive", "See Also", nullptr));
        helpSeeAlsoAction->setIconText(QCoreApplication::translate("PmTimeLive", "See Also", nullptr));
#if QT_CONFIG(whatsthis)
        helpSeeAlsoAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Help|See Also</b><br>\n"
"Click on this menu option to bring up the See Also dialog, with author and website information about PCP, Qwt, and Qt - all of which provide software infrastructure on which this tool was built.", nullptr));
#endif // QT_CONFIG(whatsthis)
        helpWhats_ThisAction->setText(QCoreApplication::translate("PmTimeLive", "What's This?", nullptr));
        helpWhats_ThisAction->setIconText(QCoreApplication::translate("PmTimeLive", "\"What's This?\" context sensitive help", nullptr));
#if QT_CONFIG(whatsthis)
        helpWhats_ThisAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Help|What's This</b><br>\n"
"Click this menu option to invoke a small question mark that is attached to the mouse pointer.  Click on a feature which you would like more information about.  A popup box appears with more information about the feature.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        helpWhats_ThisAction->setShortcut(QCoreApplication::translate("PmTimeLive", "Shift+F1", nullptr));
#endif // QT_CONFIG(shortcut)
        helpManualAction->setText(QCoreApplication::translate("PmTimeLive", "&Manual", nullptr));
        helpManualAction->setIconText(QCoreApplication::translate("PmTimeLive", "Manual", nullptr));
#if QT_CONFIG(shortcut)
        helpManualAction->setShortcut(QCoreApplication::translate("PmTimeLive", "F1", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        textLabel1->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Sampling interval</b><br>\n"
"Numeric values entered into these fields will form the number of\n"
"days, hours, minutes, seconds, and/or milliseconds between PCP\n"
"metric value samples in Time Control client programs\n"
"(e.g. pmchart, pmval, etc).", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel1->setText(QCoreApplication::translate("PmTimeLive", "Interval", nullptr));
#if QT_CONFIG(whatsthis)
        textLabel2->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Time Position</b><br>\n"
"This text field displays the current time position.  This value is\n"
"sent to clients (e.g. pmchart, pmval) of this Time Control\n"
"program to synchronise them in terms of the progression of time\n"
"and also with respect to any \"play mode\" changes requested.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel2->setText(QCoreApplication::translate("PmTimeLive", "Time", nullptr));
#if QT_CONFIG(whatsthis)
        textLabel3->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Live Control</b><br>\n"
"Pressing the Stop/Play controls will begin/block the advancement\n"
"of time, as seen by the Time Control program and all of its clients\n"
"(e.g. pmchart, pmval, etc).  Time will advance in increments of thecurrent Interval.", nullptr));
#endif // QT_CONFIG(whatsthis)
        textLabel3->setText(QCoreApplication::translate("PmTimeLive", "Control", nullptr));
#if QT_CONFIG(tooltip)
        lineEditDelta->setToolTip(QCoreApplication::translate("PmTimeLive", "Sampling interval", "Time interval between performance metric value samples"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        lineEditDelta->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Sampling interval</b><br>\n"
"Numeric values entered into these fields will form the number of\n"
"hours, minutes and/or seconds between PCP metric value\n"
"samples in Time Control client programs (e.g. pmchart, pmval, etc).", nullptr));
#endif // QT_CONFIG(whatsthis)
        comboBoxDelta->setItemText(0, QCoreApplication::translate("PmTimeLive", "Milliseconds", nullptr));
        comboBoxDelta->setItemText(1, QCoreApplication::translate("PmTimeLive", "Seconds", nullptr));
        comboBoxDelta->setItemText(2, QCoreApplication::translate("PmTimeLive", "Minutes", nullptr));
        comboBoxDelta->setItemText(3, QCoreApplication::translate("PmTimeLive", "Hours", nullptr));
        comboBoxDelta->setItemText(4, QCoreApplication::translate("PmTimeLive", "Days", nullptr));
        comboBoxDelta->setItemText(5, QCoreApplication::translate("PmTimeLive", "Weeks", nullptr));

#if QT_CONFIG(whatsthis)
        comboBoxDelta->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Sampling interval units</b><br>\n"
"Displays the units in which the sampling interval is entered.\n"
"Metric values can be sampled in days, hours, minutes, seconds\n"
"or milliseconds in Time Control client programs (e.g. pmchart,\n"
"pmval, etc).", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        lineEditCtime->setToolTip(QCoreApplication::translate("PmTimeLive", "Current date and time display", "This is the current time position as seen by all pmtime clients (e.g. pmchart, pmval, etc)"));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        lineEditCtime->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Time Position</b><br>\n"
"This text field displays the current time position.  This value is\n"
"sent to clients (e.g. pmchart, pmval) of this Time Control\n"
"program to synchronise them in terms of the progression of time\n"
"and also with respect to any \"play mode\" changes requested.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        buttonStop->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Stop</b><br>\n"
"Pressing this button will stop the advancement of time, as seen\n"
"by the Time Control program and all of its clients (e.g. pmchart,\n"
"pmval, etc).  Unless stopped, time advances in increments of the\n"
"current Interval.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonStop->setText(QString());
#if QT_CONFIG(shortcut)
        buttonStop->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(whatsthis)
        buttonPlay->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Play</b><br>\n"
"Pressing this button will begin the advancement of time, as seen\n"
"by the Time Control program and all of its clients (e.g. pmchart,\n"
"pmval, etc).  Time will advance in increments of the current\n"
"Interval.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonPlay->setText(QString());
#if QT_CONFIG(shortcut)
        buttonPlay->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        Help->setTitle(QCoreApplication::translate("PmTimeLive", "&Help", nullptr));
        File->setTitle(QCoreApplication::translate("PmTimeLive", "&File", nullptr));
        Options->setTitle(QCoreApplication::translate("PmTimeLive", "&Options", nullptr));
#if QT_CONFIG(whatsthis)
        optionsDetailAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Options|Detail</b><br>\n"
"Click this menu item to allow extended precision in time\n"
"reporting to be enabled.  Optionally, milliseconds and year can\n"
"be displayed in the current time position, and days and milliseconds\n"
"can be displayed in the current time interval.", nullptr));
#endif // QT_CONFIG(whatsthis)
        optionsDetailAction->setTitle(QCoreApplication::translate("PmTimeLive", "&Detail", nullptr));
#if QT_CONFIG(whatsthis)
        optionsTimezoneAction->setWhatsThis(QCoreApplication::translate("PmTimeLive", "<b>Options|Timezone</b><br>\n"
"Click on this menu item to display a list of available timezones,\n"
"from which to choose.  The timezone selected will be made the\n"
"new reporting timezone for all client programs (e.g. pmchart,\n"
"pmval, etc) and affects the display of timestamps in the clients\n"
"and also in the Time Control program itself.", nullptr));
#endif // QT_CONFIG(whatsthis)
        optionsTimezoneAction->setTitle(QCoreApplication::translate("PmTimeLive", "&Timezone", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PmTimeLive: public Ui_PmTimeLive {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PMTIMELIVE_H
