/********************************************************************************
** Form generated from reading UI file 'saveviewdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SAVEVIEWDIALOG_H
#define UI_SAVEVIEWDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SaveViewDialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QLabel *fileNameLabel;
    QLineEdit *fileNameLineEdit;
    QListView *dirListView;
    QHBoxLayout *hboxLayout1;
    QLabel *pathLabel;
    QComboBox *pathComboBox;
    QSpacerItem *spacerItem;
    QToolButton *parentToolButton;
    QSpacerItem *spacerItem1;
    QToolButton *userToolButton;
    QHBoxLayout *hboxLayout2;
    QVBoxLayout *vboxLayout;
    QCheckBox *preserveHostCheckBox;
    QCheckBox *preserveSizeCheckBox;
    QSpacerItem *spacerItem2;
    QVBoxLayout *vboxLayout1;
    QPushButton *savePushButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *SaveViewDialog)
    {
        if (SaveViewDialog->objectName().isEmpty())
            SaveViewDialog->setObjectName(QString::fromUtf8("SaveViewDialog"));
        SaveViewDialog->setWindowModality(Qt::WindowModal);
        SaveViewDialog->resize(414, 260);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/view.png"));
        SaveViewDialog->setWindowIcon(icon);
        SaveViewDialog->setSizeGripEnabled(true);
        gridLayout = new QGridLayout(SaveViewDialog);
#ifndef Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        fileNameLabel = new QLabel(SaveViewDialog);
        fileNameLabel->setObjectName(QString::fromUtf8("fileNameLabel"));
        fileNameLabel->setMinimumSize(QSize(60, 20));

        hboxLayout->addWidget(fileNameLabel);

        fileNameLineEdit = new QLineEdit(SaveViewDialog);
        fileNameLineEdit->setObjectName(QString::fromUtf8("fileNameLineEdit"));

        hboxLayout->addWidget(fileNameLineEdit);


        gridLayout->addLayout(hboxLayout, 2, 0, 1, 1);

        dirListView = new QListView(SaveViewDialog);
        dirListView->setObjectName(QString::fromUtf8("dirListView"));
        QFont font;
        font.setPointSize(10);
        dirListView->setFont(font);
        dirListView->setSelectionMode(QAbstractItemView::SingleSelection);
        dirListView->setIconSize(QSize(20, 20));
        dirListView->setFlow(QListView::TopToBottom);
        dirListView->setProperty("isWrapping", QVariant(true));
        dirListView->setResizeMode(QListView::Adjust);
        dirListView->setGridSize(QSize(150, 20));
        dirListView->setUniformItemSizes(true);

        gridLayout->addWidget(dirListView, 1, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(0);
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        pathLabel = new QLabel(SaveViewDialog);
        pathLabel->setObjectName(QString::fromUtf8("pathLabel"));
        pathLabel->setMinimumSize(QSize(60, 20));

        hboxLayout1->addWidget(pathLabel);

        pathComboBox = new QComboBox(SaveViewDialog);
        pathComboBox->setObjectName(QString::fromUtf8("pathComboBox"));

        hboxLayout1->addWidget(pathComboBox);

        spacerItem = new QSpacerItem(8, 26, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem);

        parentToolButton = new QToolButton(SaveViewDialog);
        parentToolButton->setObjectName(QString::fromUtf8("parentToolButton"));
        parentToolButton->setMinimumSize(QSize(20, 20));
        parentToolButton->setMaximumSize(QSize(20, 20));
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/go-previous.png"));
        parentToolButton->setIcon(icon1);
        parentToolButton->setIconSize(QSize(16, 16));
        parentToolButton->setCheckable(false);

        hboxLayout1->addWidget(parentToolButton);

        spacerItem1 = new QSpacerItem(8, 26, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem1);

        userToolButton = new QToolButton(SaveViewDialog);
        userToolButton->setObjectName(QString::fromUtf8("userToolButton"));
        userToolButton->setMinimumSize(QSize(20, 20));
        userToolButton->setMaximumSize(QSize(20, 20));
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/toolusers.png"));
        userToolButton->setIcon(icon2);
        userToolButton->setIconSize(QSize(16, 16));
        userToolButton->setCheckable(true);

        hboxLayout1->addWidget(userToolButton);


        gridLayout->addLayout(hboxLayout1, 0, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(0);
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        preserveHostCheckBox = new QCheckBox(SaveViewDialog);
        preserveHostCheckBox->setObjectName(QString::fromUtf8("preserveHostCheckBox"));
        preserveHostCheckBox->setLayoutDirection(Qt::LeftToRight);

        vboxLayout->addWidget(preserveHostCheckBox);

        preserveSizeCheckBox = new QCheckBox(SaveViewDialog);
        preserveSizeCheckBox->setObjectName(QString::fromUtf8("preserveSizeCheckBox"));

        vboxLayout->addWidget(preserveSizeCheckBox);


        hboxLayout2->addLayout(vboxLayout);

        spacerItem2 = new QSpacerItem(101, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem2);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(2);
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        savePushButton = new QPushButton(SaveViewDialog);
        savePushButton->setObjectName(QString::fromUtf8("savePushButton"));
        savePushButton->setAutoDefault(false);

        vboxLayout1->addWidget(savePushButton);

        cancelButton = new QPushButton(SaveViewDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        vboxLayout1->addWidget(cancelButton);


        hboxLayout2->addLayout(vboxLayout1);


        gridLayout->addLayout(hboxLayout2, 3, 0, 1, 1);


        retranslateUi(SaveViewDialog);

        savePushButton->setDefault(true);


        QMetaObject::connectSlotsByName(SaveViewDialog);
    } // setupUi

    void retranslateUi(QDialog *SaveViewDialog)
    {
        SaveViewDialog->setWindowTitle(QCoreApplication::translate("SaveViewDialog", "Save View", nullptr));
        fileNameLabel->setText(QCoreApplication::translate("SaveViewDialog", "Filename:", nullptr));
        pathLabel->setText(QCoreApplication::translate("SaveViewDialog", "Path:", nullptr));
#if QT_CONFIG(tooltip)
        parentToolButton->setToolTip(QCoreApplication::translate("SaveViewDialog", "Parent", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        parentToolButton->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        parentToolButton->setWhatsThis(QCoreApplication::translate("SaveViewDialog", "Open the parent directory", nullptr));
#endif // QT_CONFIG(whatsthis)
        parentToolButton->setText(QString());
        userToolButton->setText(QString());
        preserveHostCheckBox->setText(QCoreApplication::translate("SaveViewDialog", "Preserve hostnames in View", nullptr));
        preserveSizeCheckBox->setText(QCoreApplication::translate("SaveViewDialog", "Preserve window geometry in View", nullptr));
        savePushButton->setText(QCoreApplication::translate("SaveViewDialog", "Save", nullptr));
        cancelButton->setText(QCoreApplication::translate("SaveViewDialog", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SaveViewDialog: public Ui_SaveViewDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SAVEVIEWDIALOG_H
